# -*- coding: UTF-8 -*-

import config


class CashConvertersConfig(object):
    """ Класс для получения настроек конвертеров для кассы """
    def __init__(self, config, section):
        self.config = config
        self.section = section

    def getInt(self, name, defaultValue):
        return self.config.getInt(self.section, name, defaultValue)

    def getBool(self, name, defaultValue):
        return self.config.getBool(self.section, name, defaultValue)

    def getBoolWithoutDefaultValue(self, name):
        defaultValue = True
        return self.getBool(name, defaultValue)

    def getString(self, name, defaultValue):
        return self.config.getString(self.section, name, defaultValue)

    def getParamValue(self, key, defaultValue=None):
        return self.getString(key, defaultValue)

    def getFormat(self, defaultValue):
        return self.getString('fileType', defaultValue)

    def getCharset(self, converterName):
        defaultValue = 'cp1251'
        if converterName in ['smag', 'SUPERMAG', 'pilot-original', 'PILOT-ORIGINAL']:
            defaultValue = 'cp866'
        return self.getString('charset', defaultValue)

    def readSubsection(self, subsectionName):
        fullName = self.section + '.' + subsectionName;
        section = self.config.options(fullName);
        sectionMap = {}
        
        for m in section:
            sectionMap[m] = self.config.getString(fullName, m, '')
        
        return sectionMap

    def getGroup2CatalogMapper(self):
        return self.readSubsection('group2catalog')

    def getValutMapper(self):
        return self.readSubsection('valut')
    
    def getMeasureMapper(self):
        return self.readSubsection('measure')
    
    def getRolesMapper(self):
        return self.readSubsection('roles')
    
    def getPaymentMapper(self):
        return self.readSubsection('payment')

    def getOracleMeasureMappers(self):
        subSectionPath = '.converter.oracle.measure'
        sectionNames = ['upload', 'unload']
        
        mappers = dict()
        
        for s in sectionNames:
            mappers[s] = self.config.getSection(s + subSectionPath)

        return mappers
