# -*- coding: UTF-8 -*-

import codecs
import copy
import datetime
import os
import re
import string
from common import filesmodify, logger, macros

logInfo = "SHTRIH   "
id = 0
currentConverter = None
log = None
node = None
saleFiles = set()

exciseTypes = {
    'excise': {
        'storno': 163,
        'add': 160,
        'type': 0
    },
    'tobacco': {
        'storno': 183,
        'add': 180,
        'type': 5
    },
    'shoes': {
        'storno': 183,
        'add': 180,
        'type': 1520
    }
}


class SecurityLogReports(object):
    def __init__(self, securityLogs=[], cursor=0):
        self.securityLogs = securityLogs
        self.cursor = cursor
        self.current = securityLogs[cursor] if len(securityLogs) > cursor else None

    def next(self):
        self.cursor += 1
        if len(self.securityLogs) > self.cursor:
            self.current = self.securityLogs[self.cursor]
        else:
            self.current = None
        return self.current


def parseFlag(flag):
    global flagData
    f = open(flag, 'r')
    flagData = ""
    shifts = None
    dates = None
    if f.readline().strip() == '###':
        str = f.readline().strip()
        flagData = str
        if len(str.split('-')) == 2:
            if str.find('.') == -1:
                shifts = str.replace('-', '@')
            else:
                fromDate = str.split('-')[0].split('.')
                toDate = str.split('-')[1].split('.')
                dates = '%s-%s-%sT00:00:00.0@%s-%s-%sT23:59:59.99' % (fromDate[2], fromDate[1], fromDate[0], toDate[2], toDate[1], toDate[0])
    f.close()
    return shifts, dates


def openFile(log, cash, node, workDirectory, shiftNum, previousFile=None):
    global saleFiles
    output = None
    firstRound = True
    currentPath = macros.applyMacros(currentConverter.path, log, shopcode=node.id, cashcode=cash.code, shiftnum=shiftNum)

    salePath = workDirectory + '/' + os.path.basename(currentPath)
    if os.path.isfile(salePath):
        log.info(logInfo + "file exists " + salePath)

        f = codecs.open(salePath, 'r', currentConverter.charset)
        if f.readline(1) == '#':
            if not currentConverter.rewriteDestFile or previousFile == salePath:
                output = codecs.open(salePath, 'a', currentConverter.charset)
        f.next()
        try:
            f.next()
        except StopIteration:
            if output:
                output.close()
        try:
            f.next()
            firstRound = False
        except StopIteration:
            firstRound = True
        f.close()
        firstRound = False

    if not output:
        output = codecs.open(salePath, 'w', currentConverter.charset)
        output.write("#\r\n")
        output.write(unicode(str(cash.code), 'utf-8') + "\r\n")
        firstRound = True

        log.info(logInfo + "file is created " + salePath)
    saleFiles.add(output.name)
    return output, firstRound


def run(salesFunc, securityLogsFunc, shiftsFunc, cash, node1, workDirectory, currentSales):
    global currentConverter
    global log
    global node
    global saleFiles
    node = node1
    log = logger.getUnloadLogger("Node", node.id)

    log.info(logInfo + u"Запуск выгрузки продаж, касса №" + str(cash.code))
    currentConverter = node.currentConverter
    output = None
    registry = None
    registryFail = False
    firstRound = True       # Флаг. Указавет на то, чтобы писать или не писать заголовок отчета в файл.

    shifts = {}
    shiftsNotOpen = []

    for sh in shiftsFunc():
        if sh['timeBeg'] is None:
            shiftsNotOpen.append(sh['shift'])
        else:
            shifts[sh['shift']] = sh

    if shifts:
        log.info(logInfo + u"Выгрузка продаж для смен: %s" % shifts.keys())

    shift = None
    global id

    securityLogs = {}
    for secLog in securityLogsFunc():
        shiftNum = str(secLog['shiftnum'])
        if shiftNum not in securityLogs:
            securityLogs[shiftNum] = []
        if secLog['opcode'] in [8, 11]:
            securityLogs[shiftNum].append(secLog)

    securityLogReports = SecurityLogReports()
    sumDrawer = 0
    sumGain = 0

    for check in salesFunc():
        if not node.currentConverter.unloadNotOpenedShift and check['shift'] in shiftsNotOpen:
            continue
        if firstRound:
            output, firstRound = openFile(log, cash, node, workDirectory, str(check['shift']))
            if firstRound:
                output.write(str(check['shift']) + "\r\n")
                firstRound = False

        if shift != check['shift']:
            if shift in shifts:
                reports(output, None, shifts[shift], securityLogReports, sumDrawer, sumGain)
                workShiftClosing(output, shifts[shift], currentConverter.unloadShiftsFromKkm)
                previousFile = output.name
                output.close()
                output, firstRound = openFile(log, cash, node, workDirectory, str(check['shift']), previousFile)
                if firstRound:
                    output.write(str(check['shift']) + "\r\n")
                    firstRound = False
            shift = check['shift']
            id = 100000 * check['shift']
            sumDrawer = 0
            sumGain = 0
            securityLogReports = SecurityLogReports(securityLogs[str(shift)] if securityLogs.has_key(str(shift)) else [])
            if registry:
                kkmGain = 0
                for kkm in shifts[shift]['kkms']:
                    kkmGain = kkmGain + kkm['sumGain']
                registry.write(u"%s;%u;%.2f;%.2f;\r\n" % (macros.applyMacros(os.path.basename(currentConverter.path), log, shopcode = node.id, cashcode = cash.code, shiftnum = shift), shift, shifts[shift]['sumGain'], kkmGain))
            else:
                if currentConverter.saveRegistry and not registryFail:
                    log.info(logInfo + u"Создание файла журнала выгрузки")
                    if not os.path.isfile(currentConverter.registryFilePath) or currentConverter.rewriteDestFile:
                        if not os.path.isdir(os.path.dirname(currentConverter.registryFilePath)):
                            os.makedirs(os.path.dirname(currentConverter.registryFilePath))
                        registry = codecs.open(currentConverter.registryFilePath, 'w', currentConverter.charset)
                        if "flagData" in globals():
                            registry.write(flagData)
                        registry.write(u"\r\nREG_START\r\n")
                        kkmGain = 0
                        for kkm in shifts[shift]['kkms']:
                            kkmGain = kkmGain + kkm['sumGain']
                        registry.write(u"%s;%u;%.2f;%.2f;\r\n" % (macros.applyMacros(os.path.basename(currentConverter.path), log, shopcode = node.id, cashcode = cash.code, shiftnum = shift), shift, shifts[shift]['sumGain'], kkmGain))
                    else:
                        registryFail = True
                        log.info(logInfo + u"Файл %s уже существует и перезапись запрещена настройками" % (currentConverter.registryFilePath))

        if shift in shifts:
            reports(output, check, shifts[shift], securityLogReports, sumDrawer, sumGain)

        if check['docType'] == 16 or check['docType'] == 18:
            inventory(output, check)
            continue

        listDiscForCheck = []           #Все скидки
        listBonusDiscForCheck = []      #Бонусные скидки
        listBonusEarn = []              #начисления бонусов
        bonusDiscPerc = fullBonusDiscountPerc(check) #процент бунусной скидки на весь чек

        positions = check['inventPositions'] + check['stornoPositions']
        for pos in sorted(positions, key=lambda pos: pos['posNum']):
            positionsAdd(output, check, pos)
            excisePositions(output, check, pos, 'add')
            #Начисление бонусов
            if currentConverter.unloadBonuses and pos.has_key('bonusPositions'):
                listBonusEarn += bonusEarnForPosition(output, check, pos['bonusPositions'])

            if pos.has_key('timeStorno'):
                stornoPositionsRemove(output, check, pos)
                excisePositions(output, check, pos, 'storno')
            else:
                listDiscForCheck += discounts(output, pos, currentConverter.discountsDetail, check)
                #Списание бонусов
                res = bonusSpend(output, check, pos, currentConverter.discountsDetail, bonusDiscPerc)
                listDiscForCheck += res
                listBonusDiscForCheck += res

        discountCheck(output, check, listDiscForCheck)
        bonusCheck(output, check, listBonusDiscForCheck, bonusDiscPerc)

        #Начисление бонусов на чек
        if currentConverter.unloadBonuses:
            bonusEarnForCheck(output, check, listBonusEarn, listBonusDiscForCheck)

        documentPayment(output, check)

        if check['docType'] in [1, 2, 25]:  # Чек продажи/возврата/возврат по чеку
            if check['closed'] != 0:
                documentClosing(output, check)
        elif check['docType'] in [3, 4]:  # Внесения/изъятие денег
            moneyOperations(output, check)

        for moneyPos in check['moneyPositions']:
            if moneyPos['valutOperation'] == 0:
                if moneyPos['opCode'] in [44, 63,70,76,80]:
                    sumDrawer += moneyPos['sumB']
                elif moneyPos['opCode'] in [64, 72,74,82]:
                    sumDrawer -= moneyPos['sumB']
            if moneyPos['opCode'] in [63, 70,73,75,76]:
                sumGain += moneyPos['sumB']
            elif moneyPos['opCode'] in [64, 71,72,74,77]:
                sumGain -= moneyPos['sumB']

        # Если анонимная карта, выгружаем анкетные данные клиента
        for card in check['cardPositions']:
            if card['cardstatus'] != 0:
                continue
            for client in check['clientPositions']:
                if card['clientitemid'] == client['clientitemid']:
                    typeTrans = 165
                    if client['phonenumber']:
                        match = re.match("^\+7([0-9]{1})([0-9]{2})([0-9]{4})([0-9]{3})", client['phonenumber'])
                        if match:
                            tab10 = float("%s.%s" % (match.group(1), match.group(2)))
                            tab11 = float("%s.%s" % (match.group(3), match.group(4)))
                            insertData(output, check, check['timeEnd'], typeTrans, check['userCode'], card['number'], 0, tab10, tab11, 0)
                    if client['birthday']:
                        dateStr = '{0.year:4d}{0.month:02d}{0.day:02d}'.format(client['birthday'])
                        tab11 = float(dateStr[:5] + '.' + dateStr[5:])
                        insertData(output, check, check['timeEnd'], typeTrans, check['userCode'], card['number'], 1, 0, tab11, 0)
                    if client['sex'] is not None:
                        tab11 = client['sex'] + 1
                        insertData(output, check, check['timeEnd'], typeTrans, check['userCode'], card['number'], 2, 0, tab11, 0)
                    break

    if shift in shifts:
        reports(output, None, shifts[shift], securityLogReports, sumDrawer, sumGain)
        workShiftClosing(output, shifts[shift], currentConverter.unloadShiftsFromKkm)

    if output:
        output.close()
        converterSalesReadyFile = open('%s/ready' % (workDirectory), 'w')
        converterSalesReadyFile.close()
        salesReadyFile = open('%s/ready' % (os.path.dirname(os.path.dirname(workDirectory))), 'w')
        salesReadyFile.close()
        saleFiles.add(converterSalesReadyFile.name)
        saleFiles.add(salesReadyFile.name)

    if shift:
        shifts[shift] = ''

    if registry:
        registry.write(u"REG_END\r\n")
        log.info(logInfo + u"Файл журнала %s создан" % (currentConverter.registryFilePath))
        registry.close()
        saleFiles.add(registry.name)

    log.info(logInfo + u"Выгрузка выполнена")

def inventory(file, check):
    mapPosition = {}
    for pos in check['inventPositions']:

        if not pos['deptCode'] in mapPosition:
            mapPosition[pos['deptCode']] = [pos]
        else:
            mapPosition[pos['deptCode']].append(pos)

    for key in mapPosition:
        if len(mapPosition[key]):
            docType = 0
            typeTrans = 300
            if currentConverter.useTransaction300ForDocuments:
                docType = 4 # инвентаризация
                if check['docType'] == 18:
                    docType = 7 # возврат поставщику
            else:
                typeTrans = 302 # инвентаризация
                if check['docType'] == 18:
                    typeTrans = 308 # возврат поставщику

            insertData(file, check, check['timeBeg'], typeTrans, check['userCode'], '', key, check['shift']*1000+check['docNum'], docType, 0)

            # выгружаем акцизную марку в режиме инвентаризации
            if check['docType'] == 16:
                documentStore = '<Document store="%s" name1="excisemark">\r\n' % (key)
            else:
                documentStore = '<Document store="%s">\r\n' % (key)

            file.write(documentStore)

            for pos in mapPosition[key]:
                posElements = []
                posElements.append(str(pos['inventCode']))
                posElements.append(str(pos['quant']))

                if check['docType'] == 16:
                    if pos['excisemark']:
                        posElements.append(str(pos['excisemark']))
                    else:
                        posElements.append(str())

                strPosition = '%s\r\n' % str(';').join(posElements)
                file.write(strPosition)

            file.write('</Document>\r\n')

def bonusCheck(file, check, listBonusDiscForCheck, discSize):

    if not len(listBonusDiscForCheck):
        return

    userCode = check['userCode']
    posTime = ''
    cardNumber = ''
    discSum = 0

    for bonus in listBonusDiscForCheck:
        posTime = bonus['posTime']
        cardNumber = bonus['cardNumber']
        discSum += bonus['discSum']

    if check['docType'] != 1:
        discSum = -discSum
    insertData(file, check, posTime, 35, userCode, 0, 0, 0, discSize, discSum)
    insertData(file, check, posTime, 71, userCode, cardNumber, 4, 0, discSize, discSum)

def discountCheck(file, check, listCheckDisc):
    #percent = 1,
    #absolute,
    #index,
    #price,
    #bonus

    docType = check['docType']
    docBaseSum = check['baseSum']

    totalDiscount = {}
    for disc in listCheckDisc:
        if disc['discType'] == 'bonus':
            continue
        if not totalDiscount.has_key(disc['discNumber']):
            totalDiscount[disc['discNumber']] = disc
            totalDiscount[disc['discNumber']]['DISCOUNTSUM'] = disc['discSum']
            totalDiscount[disc['discNumber']]['MAXDISCSIZE'] = disc['discSize']
        else:
            totalDiscount[disc['discNumber']]['DISCOUNTSUM'] += disc['discSum']
            totalDiscount[disc['discNumber']]['MAXDISCSIZE'] = max(totalDiscount[disc['discNumber']]['MAXDISCSIZE'], disc['discSize'])

    for key in totalDiscount:
        prm11 = prm12 = abs(totalDiscount[key]['DISCOUNTSUM'])
        if totalDiscount[key]['discType'] != 'index':     # 3
            prm11 = totalDiscount[key]['MAXDISCSIZE']

        if docType != 1:
            prm12 = -abs(totalDiscount[key]['DISCOUNTSUM'])

        opCode = 37 if totalDiscount[key]['discType'] == 'percent' else 35  # 35 - суммовая, 37 - процентная скидка
        opCode += 1 if totalDiscount[key]['DISCOUNTSUM'] < 0 else 0         # 36 - суммовая, 38 - процентная надбавка

        insertData(file, check, totalDiscount[key]['posTime'], opCode, totalDiscount[key]['userCode'], 0, 0, 0, prm11, prm12)

        prm8 = prm9 = prm10 = prm11 = 0

        if totalDiscount[key]['discMode'] == 'automatic':     # 1
            prm8 = totalDiscount[key]['discScheme']
            prm9 = 2
            prm10 = totalDiscount[key]['discCode']
        elif totalDiscount[key]['discMode'] == 'byCard':      # 2
            prm8 = totalDiscount[key]['cardNumber']
            prm9 = 3
            prm10 = docBaseSum
        elif totalDiscount[key]['discMode'] == 'manual':      # 3
            prm9 = 0
        elif totalDiscount[key]['discMode'] == 'fixed':       # 3
            prm9 = 1
        elif totalDiscount[key]['discMode'] == 'bydisckit':   # 5
            prm8 = totalDiscount[key]['discScheme']
            prm9 = 2
            prm10 = totalDiscount[key]['discCode']


        if totalDiscount[key]['discMode'] == 'automatic':     # 1
            prm11 = totalDiscount[key]['MAXDISCSIZE']
        elif docBaseSum != 0:
            prm11 = abs(totalDiscount[key]['DISCOUNTSUM'] * 100 / docBaseSum )

        insertData(file, check, totalDiscount[key]['posTime'], 71, totalDiscount[key]['userCode'], prm8, prm9, prm10, prm11, prm12)

def discountPosition(file, check, disc,  inventCode, deptCode):

    docType = check['docType']
    docBaseSum = check['baseSum']

    opcode = 15
    col8 = inventCode
    col9 = deptCode
    col11 = disc['discSize']
    col12 = -abs(disc['discSum'])

    if disc['discType'] == 'percent':     # 1
        opcode = 17
    if disc['discSum'] < 0:
        opcode+=1

    if disc['discType'] in ['absolute', 'index']:     # 2, 3
        col11 = disc['discSum']

    if docType == 1:
        col12 = abs(disc['discSum'])
    insertData(file, check, disc['posTime'], opcode, disc['userCode'], col8, col9, 0, col11, col12)

    prm8 = prm9 = prm10 = prm11 = 0

    if disc['discMode'] == 'automatic':     # 1
        prm8 = disc['discScheme']
        prm9 = 2
        prm10 = disc['discCode']
    elif disc['discMode'] == 'byCard':      # 2
        prm8 = disc['cardNumber']
        prm9 = 3
        prm10 = docBaseSum
    elif disc['discMode'] == 'manual':      # 3
        prm9 = 1
    elif disc['discMode'] == 'bydisckit':   # 5
        prm8 = disc['discScheme']
        prm9 = 2
        prm10 = disc['discCode']


    if disc['discMode'] == 'automatic':     # 1
        prm11 = abs(disc['discSize'])
    elif docBaseSum != 0:
        prm11 = abs(disc['discSum'] / docBaseSum * 100)

    insertData(file, check, disc['posTime'], 70, disc['userCode'], prm8, prm9, prm10, prm11, col12)

def discounts(file, position, discountsDetail, check):

    listDiscForCheck = []
    for disc in position['discountPositions']:
        if disc['discType'] == 'bonus':
            continue
        if disc['isPositionDiscount']:
            discountPosition(file, check, disc, position['inventCode'], position['deptCode'])
        else:
            if discountsDetail:
                discountPosition(file, check, disc, position['inventCode'], position['deptCode'])
            else:
                listDiscForCheck.append(disc)
    return listDiscForCheck

# Процент скидки списания бонусов для всего чека
def fullBonusDiscountPerc(check):
    discAmount = 0
    for pos in check['inventPositions']:
        for disc in pos['discountPositions']:
            if disc['discType'] != 'bonus':
                continue
            discAmount = discAmount + disc['discSum']

    checkSum = check['docSum'] + discAmount
    result = 0 if checkSum == 0 else 100 * discAmount / checkSum
    return result

# Списанные бонусы
def bonusSpend(file, check, position, discountsDetail, bonusDiscPerc):

    docType = check['docType']
    listBonusForCheck = []
    bonusAmount = 0
    bonusPositions = []
    for disc in position['discountPositions']:
        if disc['discType'] != 'bonus':
            continue
        if disc['isPositionDiscount']:
            bonusAmount = addBonusPosition(bonusPositions, disc, docType, bonusAmount)
        else:
            if discountsDetail:
                bonusAmount = addBonusPosition(bonusPositions, disc, docType, bonusAmount)
            else:
                listBonusForCheck.append(disc)
    if len(bonusPositions):
        listBonusForCheck += bonusPositions
        bonusPosition(file, check, bonusPositions[0], bonusAmount, bonusDiscPerc)
    return listBonusForCheck

def getNotNull(dict, name, default):
    """
    Получить значение name из словаря dict
    Если в словаре нет поля name или оно None, возвращается default
    """
    result = dict.get(name, default)
    if result is None:
        result = default
    return result

def workShiftClosing(file, shift, unloadShiftsFromKkm):
    if shift['timeEnd']:
        docNum = 1
        if shift['checkNum2']:
            docNum = shift['checkNum2'] + 1

        shiftNum = 0
        sumDrawer = 0.0
        sumGain = 0.0
        if unloadShiftsFromKkm:
            if len(shift['kkms']) > 0:
                shiftNum = getNotNull(shift['kkms'][0], 'shiftnumkkm', 0)

            for kkm in shift['kkms']:
                sumDrawer += float(getNotNull(kkm, 'sumCashEnd', 0.0))
                sumGain += float(getNotNull(kkm, 'sumGain', 0.0))
        else:
            shiftNum = shift['shift']
            sumDrawer = shift['sumDrawer']
            sumGain = shift['sumGain']

        fakeCheck = {}
        fakeCheck['cashCode'] = shift['cashCode']
        fakeCheck['docNum'] = docNum
        fakeCheck['shift'] = shiftNum

        insertData(file, fakeCheck, shift['timeEnd'], 61, shift['userCode'], shiftNum, 0, sumDrawer, 0, sumGain)

def documentClosing(file, check):
    typeTrans = 55 if check['closed'] == 1 else 56
    insertData(file, check, check['timeEnd'], typeTrans, check['userCode'], 0, 0, 0, 0, check['docSum'])


def reports(file, chk, shift, securityLogReports, sumDrawer, sumGain):
    check = copy.copy(chk)

    if not check and shift['timeEnd']:
        check = {}
        check['timeBeg'] = shift['timeEnd']
        check['docNum'] = shift['checkNum2'] + 1 if shift['checkNum2'] else 1
        check['cashCode'] = shift['cashCode']
        check['shift'] = shift['shift']

    if check:
        check['docNum'] -= 1

    while securityLogReports.current:
        if check and securityLogReports.current['optime'] >= check['timeBeg']:
            break

        tmpCheck = None
        if not check:
            tmpCheck = {}
            tmpCheck['timeBeg'] = securityLogReports.current['optime']
            tmpCheck['docNum'] = shift['checkNum2']
            tmpCheck['cashCode'] = shift['cashCode']
            tmpCheck['shift'] = shift['shift']

        if securityLogReports.current['opcode'] == 8:
            sumBills = float(securityLogReports.current['extrainfo'].get('sumBills', 0.0))
            if tmpCheck is not None:
                xReport(file, tmpCheck, securityLogReports.current, sumDrawer, sumBills, sumGain)
            else:
                xReport(file, check, securityLogReports.current, sumDrawer, sumBills, sumGain)
        elif securityLogReports.current['opcode'] == 11:
            if tmpCheck is not None:
                extendedReport(file, tmpCheck, securityLogReports.current)
            else:
                extendedReport(file, check, securityLogReports.current)
        securityLogReports.next()


def xReport(file, check, op, sumDrawer, sumBills, sumGain):
    insertData(file, check, op['optime'], 60, op['cashiercard'], 0, 0, sumDrawer, sumBills, sumGain)


def extendedReport(file, check, op):
    insertData(file, check, op['optime'], 62, op['cashiercard'], 3, 0, 0, 0, 0)


def documentPayment(file, check):

    if check['closed'] != 1:
        return

    userCode = check['userCode']

    defPayment = (currentConverter.defaultCardType, 0, 0, datetime.datetime.max)
    payments = {}

    for pos in check['moneyPositions']:
        if pos['opCode'] not in [70, 72, 74, 76]:
            continue

        valCode = pos['valCode']

        # сертификат
        if pos['valutOperation'] == 3:
            sum = 0.0
            code = 0
            for cert in check['certificatePositions']:
                if cert["number"] == pos["cardnum"]:
                    sum = cert["sum"]
                    code = cert["code"]
                    break

            insertData(file, check, pos['posTime'], 122, userCode, pos["cardnum"], valCode, sum, int(code), pos['sumB'])
            isRepaid = 1 if float(pos['balance']) <= 0.001 else 0
            insertData(file, check, pos['posTime'], 73, userCode, pos["cardnum"], '', pos['sumB'], isRepaid, pos['sumB'])
            continue

        tab9, tab10, tab12, postime = defPayment
        if payments.has_key(valCode):
            tab9, tab10, tab12, postime = payments[valCode]

        if currentConverter.unloadCardType and pos['docnum'] != '':
            tab9 = pos['docnum']

        if pos['opCode'] in [72, 76]:       # Сдача
            tab10 += pos['sumB']

        if pos['opCode'] == 70:             # Сумма оплаты
            tab12 += pos['sumB']
        elif pos['opCode'] == 74:
            tab12 -= pos['sumB']

        postime = min(postime, pos['posTime'])
        payments[valCode] = (tab9, tab10, tab12, postime)

    if len(payments):
        for valCode in payments:
            payment = payments[valCode]
            tab8 = 0
            if currentConverter.unloadPaymentByCard and valCode != 1:
                tab8 = str(valCode).rjust(24, '0')
                insertData(file, check, payment[3], 95, userCode,tab8, payment[0], payment[1], valCode, payment[2])
            insertData(file, check, payment[3], 40, userCode, tab8, payment[0], payment[1], valCode, payment[2])
    elif len(check['moneyPositions']) == 0:
        insertData(file, check, check['timeEnd'], 40, userCode, 0, defPayment[0], defPayment[1], 1, defPayment[2])


def moneyOperations(file, check):
    userCode = check['userCode']
    if currentConverter.useActorCodeForMoneyOperations:
        userCode = check['moneyPositions'][0]['acode']
    typeTrans = 51
    if check['docType'] == 3:
        typeTrans = 50

    insertData(file, check, check['timeEnd'], typeTrans, userCode, 0, 0, 0, 0, check['docSum'])


def stornoPositionsRemove(file, check, invent):
    if not currentConverter.unloadPaymentCommission and invent['opCode'] in [1004, 1006]:
        return
    posSum = abs(invent['posSum'])
    code = invent['inventCode']
    typeTrans = 12

    cquant = abs(invent.get('accountingQuant', 1.000))
    modSum = abs(invent['modSum'])
    price = 0.00
    if abs(float(cquant) - 1.000) <= 0.0001:
        price = abs(invent['finalPrice'])
    else:
        price = float(modSum) / float(cquant)

    if invent['opCode'] in [50, 52, 1000, 1004]:
        posSum = -abs(invent['posSum'])
        cquant = -abs(invent['accountingQuant'])

    if invent['opCode'] in [52, 56, 60]:
        code = ''
        typeTrans = 2

    insertData(file, check, invent['timeStorno'], typeTrans, invent["userCode"], code, invent['deptCode'], "%.2f" % (price), cquant, posSum)


def getInventExciseType(inventTags):
    """ Получение типа товара """
    tags = []
    if inventTags:
        tags = inventTags.split(";")

    if 'excise' in tags:
        return 'excise'
    elif 'tobacco' in tags:
        return 'tobacco'
    elif 'shoes' in tags:
        return 'shoes'
    return ''


def excisePositions(file, check, invent, op):
    # проверка наличия акцизной марки(кода маркировки)
    if not invent['excisemark']:
        return

    inventExciseType = getInventExciseType(invent['tags'])
    if not inventExciseType or (not currentConverter.unloadExciseMark and inventExciseType == 'excise'):
        return
    typeTransaction = exciseTypes[inventExciseType][op]
    excisemarktype = exciseTypes[inventExciseType]['type']
    # Из документации: при выгрузке транзакций в файл поля разделяются символом «;» (точка с запятой),
    # в поле 8 символ «;» заменяется на последовательность символов «\u003B» (без кавычек).
    insertData(file, check, invent['posTime'], typeTransaction, invent["userCode"],
               invent['excisemark'].replace(';', '\u003B'), 0, excisemarktype, 0, 0)


def positionsAdd(file, check, invent):
    if not currentConverter.unloadPaymentCommission and invent['opCode'] in [1004, 1006]:
        return
    typeTransaction = None
    code = invent['inventCode']
    cquant = abs(invent.get('accountingQuant', 1.000))
    posSum = abs(invent['modSum'])

    price = 0.00
    if abs(float(cquant) - 1.000) <= 0.0001:
        price = abs(invent['finalPrice'])
    else:
        price = float(posSum) / float(cquant)

    if not invent['opCode'] in [50, 52, 63, 1000, 1004, 1011]:
        cquant = -abs(invent['accountingQuant'])
        posSum = -abs(invent['modSum'])

    if invent['opCode'] in [52, 56, 60]:
        code = ''

    if invent['opCode'] in [50, 1000, 1004, 1011]:
        typeTransaction = 11
    elif invent['opCode'] == 52:
        typeTransaction = 1
    elif invent['opCode'] == 54:
        typeTransaction = 14
    elif invent['opCode'] == 56:
        typeTransaction = 4
    elif invent['opCode'] == 58:
        typeTransaction = 13
    elif invent['opCode'] == 60:
        typeTransaction = 3
    elif invent['opCode'] == 63:
        typeTransaction = 72
    elif invent['opCode'] == 64:
        typeTransaction = 74

    userCode = invent["userCode"]
    if invent['opCode'] in [63, 64]: # продажа/возврат сертификата
        if currentConverter.unloadInventTransactionForCertificate:
            posTransaction = 11 if typeTransaction == 72 else 13
            insertData(file, check, invent['posTime'], posTransaction, userCode, invent["inventCode"], invent['deptCode'], "%.2f" % (price), cquant, posSum)
        insertData(file, check, invent['posTime'], typeTransaction, userCode, invent["barCode"], '', "%.2f" % (price), 0, "%.2f" % (price))
        return

    if (typeTransaction == 13 or (currentConverter.useCheckCashierCodeForStorno and typeTransaction == 11 and invent.has_key('timeStorno'))):
        userCode = check["userCode"]

    insertData(file, check, invent['posTime'], typeTransaction, userCode, code, invent['deptCode'], "%.2f" % (price), cquant, posSum)
    if (invent.has_key('aspectPosition') and len(invent['aspectPosition']) > 0):
        insertData(file, check, invent['posTime'], 20, userCode, invent['aspectPosition'][0]['productsectioncode'], invent['deptCode'], None, None, None)
    if abs(float(abs(invent.get('accountingQuant', 1.000))) - 1.000) > 0.0001:
        insertData(file, check, invent['posTime'], 24, userCode, invent['barCode'], invent['deptCode'], 0.00, cquant, 0.00)

    # добавление консультанта в позицию(транзакция 90)
    if invent.has_key('consultantid') and invent['consultantid']:
        insertData(file, check, invent['posTime'], 90, userCode, invent['consultantid'], invent['deptCode'], None, None, None)


def addBonusPosition(bonusPositions, disc, docType, amount):
    if disc['discType'] != 'bonus':
        return amount
    if docType not in [1, 2, 25]:
        return amount
    amount = amount + disc['discSum']
    bonusPositions.append(disc)
    return amount


def bonusPosition(file, check, disc, amount, bonusDiscPerc):

    typeTrans = 162
    sum1 = amount

    if check['docType'] != 1:
        typeTrans = 164
        sum1 = -amount

    insertData(file, check, disc['posTime'], typeTrans, check['userCode'], disc['cardNumber'], 0, 0, bonusDiscPerc, sum1)


def bonusEarnForPosition(file, check, bonusPositions):
    """ Добавляет начисления баллов на позицию """
    docType = check['docType']
    userCode = check['userCode']

    if docType not in [1, 2, 25]:
        return []

    resultBonusPos = None
    for bonusPosition in bonusPositions:
        if bonusPosition['ispositionbonus']:
            if resultBonusPos:
                resultBonusPos['amount'] += bonusPosition['amount']
            else:
                resultBonusPos = bonusPosition

    if resultBonusPos:
        if docType in [2, 25]:
            resultBonusPos['amount'] = -resultBonusPos['amount']
        typeTrans = 161 if docType == 1 else 163
        insertData(file, check, resultBonusPos['posTime'], typeTrans, userCode, resultBonusPos['cardNumber'], 0, 0, 0, resultBonusPos['amount'])

    return bonusPositions


def bonusEarnForCheck(file, check, bonusPositions, listBonusDiscForCheck):
    """
    Метод добавляет информацию о начислении баллов на чек. Если есть начисления на чек,
    все эти начисления группируются по полю 'bonusnumber' и добавляются в виде 161 или 163 транзакции.
    Если нет начислений на чек, но есть начисления на позиции или списания баллов,
    также добавляется транзакция, но с нулевой суммой.
    9 поле - признак бонусного начисления на чек
    10 поле - номер бонусной акции
    """
    docType = check['docType']
    userCode = check['userCode']

    if docType not in [1, 2, 25]:
        return

    amountFunc = lambda x: x
    if docType in [2, 25]:
        amountFunc = lambda x: -x

    actions = {}
    # Собираем все акции на чек из позиционных
    for bonusPosition in bonusPositions:
        if not bonusPosition['ispositionbonus']:
            bonusnumber = bonusPosition['bonusnumber']
            if actions.has_key(bonusnumber):
                actions[bonusnumber]['amount'] += bonusPosition['amount']
            else:
                actions[bonusnumber] = bonusPosition

    typeTrans = 161 if docType == 1 else 163

    if len(actions):
        for bonusnumber in actions:
            action = actions[bonusnumber]
            insertData(file, check, action['posTime'], typeTrans, userCode, action['cardNumber'], 1,  action['campaignid'], 0, amountFunc(action['amount']))
    elif len(bonusPositions) or len(listBonusDiscForCheck):
        posTime = bonusPositions[0]['posTime'] if len(bonusPositions) else listBonusDiscForCheck[0]['posTime']
        cardNumber = bonusPositions[0]['cardNumber'] if len(bonusPositions) else listBonusDiscForCheck[0]['cardNumber']
        insertData(file, check, posTime, typeTrans, userCode, cardNumber, 1, 0, 0, 0)
    else:
        for card in check['cardPositions']:
            insertData(file, check, card['posTime'], typeTrans, userCode, card['number'], 1, 0, 0, 0)


def insertData(file, check, transDateTime, transType, userCode, tab8, tab9, tab10, tab11, tab12):
    cashCode = check['cashCode']
    docNum = check['docNum']
    shiftNum = check['shift']

    if tab8 == None:
        tab8 = 0
    if tab9 == None:
        tab9 = 0
    if tab10 == None:
        tab10 = 0
    if tab11 == None:
        tab11 = 0.0
    if tab12 == None:
        tab12 = 0

    tab11 = round(tab11, 3)
    if transType == 160:
        tab11 = 0
    global id
    id += 1
    transNum = str(id)
    if transType in [300, 302, 308]:
        transNum = '2' + transNum
    transDate = transDateTime.strftime('%d.%m.%Y')
    if currentConverter.unloadShortDateFormat:
        transDate = transDateTime.strftime('%d.%m.%y')
    transTime = transDateTime.strftime('%H:%M:%S')

    if currentConverter.checkNumTransaction:
        docNum = macros.applyMacros(currentConverter.checkNumTransaction, log, shopcode=node.id, cashcode=cashCode, shiftnum=shiftNum, checknum=docNum)

    strTrans = '%s$dl%s$dl%s$dl%s$dl%s$dl%s$dl%s$dl%s$dl%s$dl%s$dl%s$dl%s$dl\r\n' % (transNum, transDate, transTime, transType, cashCode, docNum, userCode, tab8, tab9, tab10, tab11, tab12)
    strTransWithDelimiter = string.replace(strTrans, '$dl', currentConverter.delimiter)
    file.write(strTransWithDelimiter)


def sendSales(cash, node, workDirectory, backupDir):
    log = logger.getUnloadLogger("Node", node.id)
    rewriteDestFile = filesmodify.checkFilesInRemoteDirectory(rewrite=node.currentConverter.rewriteDestFile,
            workDirectory=workDirectory, remoteDirectory=str(os.path.dirname(node.currentConverter.path)))
    return filesmodify.safeMoveDirectory(log, workDirectory, os.path.dirname(node.currentConverter.path),
            backupDir=backupDir, rewrite=rewriteDestFile, maxCountFiles=cash.maxCountFiles, actualCountDays=cash.actualCountDays)


def sendOnlineSales(cash, node, workDirectory):
    return False, []


def getSaleFiles():
    global saleFiles
    return saleFiles
