# -*- coding: UTF-8 -*-

import codecs
import os


class DataHeader(object):
    def __init__(self, workDir, charset, currentSales):
        """
        @param currentSales - выгрузка текущих продаж
        """
        self.workDirectory = workDir
        self.currentSales = currentSales

        self.cashSail = self.openFile('cashSail.dat', charset)
        self.cashDcrd = self.openFile('cashDcrd.dat', charset)
        self.saleFiles = set((self.cashSail.name, self.cashDcrd.name))
        if not self.currentSales:
            self.cashDisc = self.openFile('cashDisc.dat', charset)
            self.currests = self.openFile('currests.dat', charset)
            self.curMoney = self.openFile('curMoney.dat', charset)
            self.cashPay = self.openFile('cashPay.dat', charset)
            self.saleFiles.add(self.cashDisc.name)
            self.saleFiles.add(self.currests.name)
            self.saleFiles.add(self.curMoney.name)
            self.saleFiles.add(self.cashPay.name)

    def openFile(self, file, charset):
        """ Открывает файл. Создаем новый или дописываем """
        pathFile = os.path.normpath(self.workDirectory + '/' + file)
        arg = 'a' if os.path.exists(pathFile) else 'w'
        return codecs.open(pathFile, arg, charset)

    def close(self):
        self.cashSail.close()
        self.cashDcrd.close()
        if not self.currentSales:
            self.cashDisc.close()
            self.currests.close()
            self.curMoney.close()
            self.cashPay.close()

    def setCashSail(self, SHOPINDEX, CASHNUMBER, ZNUMBER, CHECKNUMBE, ID, DATE, TIME, CARDARTICU, QUANTITY, PRICERUB, TOTALRUB, CASHER, USINGINDEX, REPLACE, OPERATION, CREDCARDIN):
        dat = DATE.strftime("%d/%m/%Y")
        strOutput = "%s,%s,%s,%s,%s,%s,%s,\"%s\",\"%s\",%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s\n" % (SHOPINDEX, CASHNUMBER, ZNUMBER, CHECKNUMBE, ID, dat, TIME, CARDARTICU, 'NOSIZE', QUANTITY, PRICERUB, 0, TOTALRUB, 0, 0, CASHER, USINGINDEX, REPLACE, OPERATION, CREDCARDIN, 0, 0)
        self.cashSail.write(strOutput)

    def setCashDisc(self, SHOPINDEX, CASHNUMBER, ZNUMBER, CHECKNUMBE, ID, DISCOUNTIN, DISCOUNTPR, DISCOUNTRU):
        if self.currentSales:
            return
        strOutput = "%s,%s,%s,%s,%s,%s,%.2f,%s,%s\n" % (SHOPINDEX, CASHNUMBER, ZNUMBER, CHECKNUMBE, ID, DISCOUNTIN, DISCOUNTPR, DISCOUNTRU, 0)
        self.cashDisc.write(strOutput)

    def setCurrests(self, counterSales):
        if self.currentSales:
            return
        for shiftNum in counterSales:
            for cashCode in counterSales[shiftNum]:
                value = counterSales[shiftNum][cashCode]

                dat = value['DATE'].strftime("%d/%m/%Y")

                strOutput = "%s,%s,%s,%s,%s,%s,%s,%s,%s,%s\n" % (value['SHOPINDEX'], cashCode, shiftNum, dat, value['SALE'], value['RETURN'], 0, 0, value['SALE'] - value['RETURN'], 0)
                self.currests.write(strOutput)

    def setCurMoney(self, cashDR):
        if self.currentSales:
            return
        for shiftNum in cashDR:
            for cashCode in cashDR[shiftNum]:
                value = cashDR[shiftNum][cashCode]

                strOutput = "%s,%s,%s,%s,%s,%s,%s,%s,%s,%s\n" % (value['SHOPINDEX'], cashCode, shiftNum, 0, value['BEFORE'], 0, 0, value['PERIODIN'], value['PERIODOUT'], value['AFTER'])
                self.curMoney.write(strOutput)

    def setCashDcrd(self, SHOPINDEX, CASHNUMBER, ZNUMBER, CHECKNUMBE, CARDNUMBER, DISCOUNTRU):
        strOutput = "%s,%s,%s,%s,%s,\"%s\",%s,%s\n" % (SHOPINDEX, CASHNUMBER, ZNUMBER, CHECKNUMBE, 0, CARDNUMBER, DISCOUNTRU, 0)
        self.cashDcrd.write(strOutput)

    def setCashPay(self, SHOPINDEX, CASHNUMBER, ZNUMBER, CHECKNUMBE, PAYMENT, PAYEDMONEY):
        if self.currentSales:
            return
        strOutput = "%s,%s,%s,%s,%s,\"%s\",%s,%s,%s,%s,%s,%s,%s,%s\n" % (SHOPINDEX, CASHNUMBER, ZNUMBER, CHECKNUMBE, PAYMENT, '', PAYEDMONEY, 0, PAYEDMONEY,0,0,0,0,0)
        self.cashPay.write(strOutput)

    def getSaleFiles(self):
        return self.saleFiles


class CurrentSalesDataHeader(object):
    def __init__(self, workDir, charset, cashSailFile, cashDcrdFile):
        self.workDirectory = workDir

        files = [cashSailFile, cashDcrdFile]

        dict = {}
        for file in files:
            pathFile = os.path.normpath(self.workDirectory + '/' + file)
            dict[file] = pathFile

        self.cashSail = codecs.open(dict[cashSailFile], 'w', charset)
        self.cashDcrd = codecs.open(dict[cashDcrdFile], 'w', charset)

    def close(self):
        self.cashSail.close()
        self.cashDcrd.close()

    def setCashSail(self, SHOPINDEX, CASHNUMBER, ZNUMBER, CHECKNUMBE, ID, DATE, TIME, CARDARTICU, NAME, QUANTITY, PRICERUB, TOTALRUB, CASHER, USINGINDEX, REPLACE, OPERATION, CREDCARDIN):

        dat = DATE.strftime("%d/%m/%Y")

        strOutput = "%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s\n" % (SHOPINDEX, CASHNUMBER, ZNUMBER, CHECKNUMBE, ID, dat, TIME, CARDARTICU, NAME, 'NOSIZE', QUANTITY, PRICERUB, 0, TOTALRUB, 0, 0, CASHER, USINGINDEX, REPLACE, OPERATION, CREDCARDIN, 0, 0)
        self.cashSail.write(strOutput)

    def setCashDcrd(self, SHOPINDEX, CASHNUMBER, ZNUMBER, CHECKNUMBE, CARDNUMBER, DISCOUNTRU):

        strOutput = "%s,%s,%s,%s,%s,%s,%s,%s\n" % (SHOPINDEX, CASHNUMBER, ZNUMBER, CHECKNUMBE, 0, CARDNUMBER, DISCOUNTRU, 0)
        self.cashDcrd.write(strOutput)
