# -*- coding: UTF-8 -*-
import base64
import httplib2
import json
import os
import sys
import time
from urlparse import urlparse

from common import config, logger, macros
from common.csconvertersconfig import CSConvertersConfig
from unload.node.convertergetter import getConverterByName
from unload.node.node import Node
from unload.node.nodeparams import DbReader, Cash

log = logger.getUnloadLogger("main")

jsonHeader = {'Content-Type': 'application/json', "Accept": "application/json"}
requestTimeout = 60


def getConverter(converterName, csConverterConfig, **kwargs):
    converter = None
    salePath = csConverterConfig.getString('salePath', '../sales/%(shopcode)s/%(cashcode)s')
    saleFile = macros.applyMacrosWithoutTime(salePath, log, shopcode=kwargs.get('shopCode', None),
            cashcode=kwargs.get('cashCode', None), shopid=kwargs.get('shopId', None),
            cashid=kwargs.get('cashId', None), clusterid=kwargs.get('clusterId', None))

    onlineSalePath = csConverterConfig.getString('onlineSalePath', '')
    onlineSaleWithMacro = None
    if onlineSalePath:
        onlineSaleWithMacro = macros.applyMacrosWithoutTime(str(onlineSalePath), log,
                shopcode=kwargs.get('shopCode', None), cashcode=kwargs.get('cashCode', None),
                shopid=kwargs.get('shopId', None), cashid=kwargs.get('cashId', None),
                clusterid=kwargs.get('clusterId', None))

    rewriteDestFile = csConverterConfig.getBool('rewriteDestFile', False)
    flagPath = csConverterConfig.getString('saleFlag', os.path.join(os.path.dirname(saleFile), 'pos%(shopcode)s.flr'))
    flagFile = macros.applyMacrosWithoutTime(str(flagPath), log, shopcode=kwargs.get('shopCode', None),
            cashcode=kwargs.get('cashCode', None), shopid=kwargs.get('shopId', None),
            cashid=kwargs.get('cashId', None), clusterid=kwargs.get('clusterId', None))

    converter = getConverterByName(converterName, csConverterConfig, saleFile, flagFile, onlineSaleWithMacro, rewriteDestFile)
    if converter:
        converter.backup_dir = kwargs.get('backupDir', None)
    return converter


def getNodes():
    cs_host = 'localhost'
    cs_port = '8080'
    cs_prefix = '/CSrest/rest'
    cs_login = 'admin'
    cs_password = 'admin'
    db_connection = 'mysql://netroot:netroot@localhost/%(CLUSTER)sdocuments%(DB_SUFFIX)s'

    if not config.read(sys.path[0] + "/../config/%s/exchangers.ini" % logger.location):
        log.warning("Configuration file 'exchangers.ini' not found. Configure backup properties by default...")
    else:
        maxCountFiles = 30
        actualCountDays = 30
        maxCountFiles = config.getInt('UNLOAD', 'maxCountBackupFiles', maxCountFiles)
        actualCountDays = config.getInt('UNLOAD', 'actualCountDaysForBackupFiles', actualCountDays)
        cs_host = config.getString('CS', 'node.host', cs_host)
        cs_port = config.getString('CS', 'node.port', cs_port)
        cs_prefix = config.getString('CS', 'node.prefix', cs_prefix)
        cs_login = config.getString('CS', 'login', cs_login)
        cs_password = config.getString('CS', 'passwd', cs_password)
        db_connection = (config.getString('UNLOAD', 'dbConnection', db_connection))

    base_url = 'http://' + cs_host + ':' + cs_port + cs_prefix
    auth = base64.b64encode('{0}:{1}'.format(cs_login, cs_password).encode('utf-8')).decode()
    jsonHeader['Authorization'] = 'Basic %s' % auth

    serverMode = {}
    # если на момент старта системы КС еще не доступен (еще стартует) - попробуем сделать запрос несколько раз
    attemptCounter = 30
    http = httplib2.Http(timeout=requestTimeout)
    while attemptCounter > 0:
        try:
            response, content = http.request(base_url + '/system/mode', 'GET', headers=jsonHeader)
            if response.status == 200:
                serverMode = json.loads(content)
                log.info("Server available and answered")
                break
        except Exception as e:
            log.warn("CS response exception: %s", e.message)
        log.warn("Can't get response from: %s. Wait 10 sec and try again... attempts remaining: %s", base_url, attemptCounter)
        time.sleep(10)
        attemptCounter -= 1

    serverInfo = {}
    response, content = http.request(base_url + '/system/info', 'GET', headers=jsonHeader)
    if response.status == 200:
        serverInfo = json.loads(content)

    allCashes = []
    response, content = http.request(base_url + '/cashes', 'GET', headers=jsonHeader)
    if response.status == 200:
        allCashes = json.loads(content)

    response, content = http.request(base_url + '/shops', 'GET', headers=jsonHeader)
    if response.status != 200:
        return []

    shops = json.loads(content)

    nodes = []
    for shop in shops:
        shopCode = str(shop.get('shopCode'))
        shopId = str(shop.get('id'))
        clusterId = str(shop.get('clusterId', ''))

        cashes = []
        findedCashes = [item for item in allCashes if item.get('parentShop') == shopId]
        for cash in findedCashes:
            converters = []
            cashCode = str(cash.get('cashCode'))
            cashId = str(cash.get('id'))
            saleExchangeConfigs = cash.get('exchange', {}).get('saleConfigs', {})
            for saleExchangeConfig in saleExchangeConfigs:
                saleExchangeTypeName = saleExchangeConfig.get('type', 'DEFAULT') if saleExchangeConfig else 'DEFAULT'
                if saleExchangeTypeName in ['DEFAULT', 'ARTIXINTERCHANGE_SOAP', 'ARTIXINTERCHANGE_REST']:
                    continue
                csConverterConfig = CSConvertersConfig(saleExchangeConfig)
                backupPath = csConverterConfig.getString('backupDir', '../backup/sales/%(shopcode)s/%(cashcode)s')
                backupDir = macros.applyMacrosWithoutTime(str(backupPath), log, shopcode=shopCode,
                        cashcode=cashCode, shopid=shopId, cashid=cashId, clusterid=clusterId)
                converter = getConverter(saleExchangeTypeName, csConverterConfig, shopCode=shopCode,
                        cashCode=cashCode, shopId=shopId, cashId=cashId, clusterId=clusterId, backupDir=backupDir)

                converters.append(converter)
            if len(converters) > 0:
                cashes.append(Cash(cashCode, converters, backup_dir=backupDir, maxCountFiles=maxCountFiles,
                        actualCountDays=actualCountDays, cashId=cashId))

        dbType = 'mysql'
        dbPortDefault = 3306
        dbSuffix = 'All'
        salesLoadModes = serverInfo.get('salesLoadModes', ['GLOBAL_MYSQL'])
        if 'STORE_MYSQL' in salesLoadModes:
            dbSuffix = shopId
            if str(serverMode.get('hierarhyMode')) == 'CENTRAL' and str(shop.get('parentObject', {}).get('parentType')) == 'MIDDLE_SERVER':
                dbSuffix = str(shop.get('parentObject', {}).get('parentCode'))
        elif 'GLOBAL_PSQL' in salesLoadModes and not 'GLOBAL_MYSQL' in salesLoadModes:
            dbType = 'psql'
            dbPortDefault = 5432

        connectionParams = urlparse(db_connection)
        nodes.append(Node(id=shopId,
                          cashes=cashes,
                          reader=DbReader(
                              host=connectionParams.hostname if connectionParams.hostname else 'localhost',
                              port=connectionParams.port if connectionParams.port else dbPortDefault,
                              user=connectionParams.username if connectionParams.username else 'netroot',
                              passwd=connectionParams.password if connectionParams.password else 'netroot',
                              db=((connectionParams.path if connectionParams.path else '%(CLUSTER)sdocuments%(DB_SUFFIX)s') %
                                  {"CLUSTER": clusterId, "DB_SUFFIX": dbSuffix})[1:],
                              dbtype=dbType

                          ),
                          shopCode=shopCode))
    return nodes
