# -*- coding: UTF-8 -*-

import re
from common.parameters import DatabaseParameters


class DbReader(DatabaseParameters):
    def __init__(self, host, port, user, passwd, db, dbtype='mysql'):
        self.dbtype = dbtype
        DatabaseParameters.__init__(self, host, port, user, passwd, db)


class DictConverter(object):
    def __init__(self, name, path, flag=None, backup_dir=None):
        self.name = name                # name of converter
        self.path = path                # filename
        self.flag = flag                # file-flag for unload
        self.backup_dir = backup_dir


class SmagConverter(DictConverter):
    def __init__(self, path, format, valutMapper={}, flag=None, rewriteDestFile=False, onlineSalePath=None,
            charset='cp1251', 
            appendSaleFiles=False, distributePaymentsForPositions=False):
        DictConverter.__init__(self, 'smag', path, flag)
        self.format = format            # dat | dbf
        self.rewriteDestFile = rewriteDestFile
        self.onlineSalePath = onlineSalePath
        self.charset = charset
        self.appendSaleFiles = appendSaleFiles
        self.valutMapper = valutMapper
        self.distributePaymentsForPositions = distributePaymentsForPositions


class AifConverter(DictConverter):
    def __init__(self, path, flag=None, rewriteDestFile=False, dataType=None, withClosedShifts=True,
            onlineSalePath = "/tmp/salesOnline/"):
        DictConverter.__init__(self, 'aif', path, flag)
        self.rewriteDestFile = rewriteDestFile
        self.dataType = 'sales'
        if dataType in ['sales', 'crights', 'all']:
            self.dataType = dataType        # unload data type: sales, crights, all
        self.withClosedShifts = withClosedShifts
        self.onlineSalePath = onlineSalePath


class ShtrihConverter(DictConverter):
    def __init__(self, path, discountsDetail=False, flag=None, rewriteDestFile=False,
            useActorCodeForMoneyOperations=False, unloadShortDateFormat=False, unloadCardType=True,
            defaultCardType = '', charset='cp1251', delimiter=';', unloadPaymentByCard=False, unloadBonuses=False,
            unloadPaymentCommission=False, useTransaction300ForDocuments=True, unloadExciseMark=False,
            unloadShiftsFromKkm=False, checkNumTransaction='', unloadNotOpenedShift=False,
            useCheckCashierCodeForStorno=False, unloadInventTransactionForCertificate=True,
            saveRegistry=False, registryFilePath='/tmp/sales/sales.reg'):
        DictConverter.__init__(self, 'shtrih', path, flag)
        self.discountsDetail = discountsDetail              # detail the discount on the check or not
        self.rewriteDestFile = rewriteDestFile
        self.useActorCodeForMoneyOperations = useActorCodeForMoneyOperations
        self.unloadShortDateFormat = unloadShortDateFormat
        self.unloadCardType = unloadCardType
        self.defaultCardType = defaultCardType if defaultCardType else ''
        self.charset = charset
        if delimiter:
            p = re.compile('(\[)([0-9]+)(\])')
            m = p.match(delimiter)
            if m:
                delimiter = chr(int(m.group(2)))
        else:
            delimiter = ';'
        self.delimiter = delimiter
        self.unloadPaymentByCard = unloadPaymentByCard
        self.unloadBonuses = unloadBonuses
        self.unloadPaymentCommission = unloadPaymentCommission
        self.useTransaction300ForDocuments = useTransaction300ForDocuments
        self.unloadExciseMark = unloadExciseMark
        self.unloadShiftsFromKkm = unloadShiftsFromKkm
        self.checkNumTransaction = checkNumTransaction
        self.unloadNotOpenedShift = unloadNotOpenedShift
        self.useCheckCashierCodeForStorno = useCheckCashierCodeForStorno
        self.unloadInventTransactionForCertificate = unloadInventTransactionForCertificate
        self.saveRegistry = saveRegistry
        self.registryFilePath = registryFilePath


class AtolAstorConverter(DictConverter):
    def __init__(self, path, discountsDetail=False, flag=None, rewriteDestFile=False,
            clearUnloadDirectory=False, charset='cp1251', onlineSalePath="/tmp/salesOnline/"):
        DictConverter.__init__(self, 'atol-astor', path, flag)
        self.discountsDetail = discountsDetail              # detail the discount on the check or not
        self.rewriteDestFile = rewriteDestFile
        self.charset = charset
        self.onlineSalePath = onlineSalePath


class AtolConverter(DictConverter):
    def __init__(self, path, discountsDetail=False, flag=None, rewriteDestFile=False,
            clearUnloadDirectory=False, charset='cp1251', onlineSalePath="/tmp/salesOnline/"):
        DictConverter.__init__(self, 'atol', path, flag)
        self.discountsDetail = discountsDetail              # detail the discount on the check or not
        self.rewriteDestFile = rewriteDestFile
        self.charset = charset
        self.onlineSalePath = onlineSalePath


class OracleConverter(DictConverter):
    def __init__(self, path, flag=None, rewriteDestFile=False, measuresMapper={}, valutMapper={}):
        DictConverter.__init__(self, 'oracle', path, flag)
        self.rewriteDestFile = rewriteDestFile
        self.measuresMapper = measuresMapper
        self.valutMapper = valutMapper


class CrystalConverter(DictConverter):
    def __init__(self, path, flag=None, rewriteDestFile=False, formdiscounts=True, paymentMapper={}):
        DictConverter.__init__(self, 'crystal', path, flag)
        self.rewriteDestFile = rewriteDestFile
        self.formdiscounts = formdiscounts
        self.paymentMapper = paymentMapper


class NS2000Converter(DictConverter):
    def __init__(self, path, flag=None, rewriteDestFile=False, charset='cp866'):
        DictConverter.__init__(self, 'ns2000', path, flag)
        self.rewriteDestFile = rewriteDestFile
        self.charset = charset


class PilotConverter(DictConverter):
    def __init__(self, path, flag=None, rewriteDestFile=False, charset='cp866'):
        DictConverter.__init__(self, 'pilot', path, flag)
        self.rewriteDestFile = rewriteDestFile
        self.charset = charset


class Cash(object):
    def __init__(self, code, converters=[], backup_dir=None, maxCountFiles=30, actualCountDays=30, cashId=None):
        self.code = code            # code of cash
        self.converters = converters
        self.backup_dir = backup_dir
        self.maxCountFiles = maxCountFiles
        self.actualCountDays = actualCountDays
        self.cashId = cashId if cashId else code


class SapConverter(DictConverter):
    def __init__(self, path, flag=None, rewriteDestFile=False, onlineSalePath="/tmp/salesOnline/", rootString=""):
        DictConverter.__init__(self, 'sap', path, flag)
        self.rewriteDestFile = rewriteDestFile
        self.onlineSalePath = onlineSalePath
        self.rootString = rootString


class PilotOriginalConverter(DictConverter):
    def __init__(self, path, flag=None, rewriteDestFile=False, charset='cp866'):
        DictConverter.__init__(self, 'pilot-original', path, flag)
        self.rewriteDestFile = rewriteDestFile
        self.charset = charset
