# -*- coding: UTF-8 -*-

import os
import sys
sys.path[0] = sys.path[0] + '/../../..'

from common import logger
logger.location = 'cs'

from common import config, macros
from artix.common import servicelock
from unload.service.reader import Reader
import optparse
import atexit
import tempfile
import signal
import traceback

log = logger.getUnloadLogger("main")
lockFile = os.path.normpath('%s/locks/lockArtixStartReaderCS' % tempfile.gettempdir())
srvLock = servicelock.ServiceLock(lockFile, log)
atexit.register(srvLock.unlock)
signal.signal(signal.SIGTERM, srvLock.unlock)
r = Reader()


def unload(node, workDirectory, request=None, converterName=None, shifts=None, dates=None, granularity='document',
        closedShifts=0, changedShifts=False, cashCode=None, cashUnloadStateId=None, statusHost=None, isSendDebugStatus=True):
    log = logger.getUnloadLogger("Node", node.id)
    unloadNowFile = workDirectory+'/%s/unload/%s-unloadNow%s' % (node.id, cashCode,  "-%s" % converterName if converterName else '')

    srvLockUnload = servicelock.ServiceLock(unloadNowFile, log)
    atexit.register(srvLockUnload.unlock)
    try:
        signal.signal(signal.SIGTERM, srvLockUnload.unlock)
    except:
        pass

    if srvLockUnload.lock(False):
        try:
            if request and converterName:
                log.info('unload by request %s' % request)
                shifts = None
                dates = None
                try:
                    plugin = __import__('unload.converter.%s.%s' % (converterName, converterName), fromlist='q')
                    shifts, dates = plugin.parseFlag(request)
                except Exception, e:
                    log.error(e)
                    log.error(traceback.format_exc())
                log.info('remove file-flag %s' % request)
                os.remove(request)
                r.read(node, closedShifts, shifts, dates, "document" if not shifts and not dates else "shift",
                        False, workDirectory, converterName=converterName, 
                        cashCode=cashCode, cashUnloadStateId=cashUnloadStateId, statusHost=statusHost, isSendDebugStatus=isSendDebugStatus)
            else:
                r.read(node, closedShifts, shifts, dates, granularity, changedShifts, workDirectory, converterName=converterName, 
                       cashCode=cashCode, cashUnloadStateId=cashUnloadStateId, statusHost=statusHost, isSendDebugStatus=isSendDebugStatus)
            if os.path.exists(workDirectory+'/%s/unload/ready' % node.id):
                r.send(node, workDirectory)
        except Exception, e:
            log.error(str(e))
        finally:
            srvLockUnload.unlock()
        return True
    else:
        log.info('Can not start unload. Unloading is already running...')
        return False


def applyConverterAndPathOptions(node, cash, converterName, dataPath):
    if converterName:
        if not any(x.name == converterName for x in cash.converters):
            log.warn(u"Касса %s не поддерживает тип конвертера %s" % (cash.cashId, converterName))
            return False
        # оставляем только нужные конвертеры
        cash.converters = [converter for converter in cash.converters if converter.name == converterName]
    if dataPath:
        for converter in cash.converters:
            converter.path = macros.applyMacros(dataPath, log, shopid=node.id, shopcode=node.shopCode, cashcode=cash.code, cashid=cash.cashId)
    return True


def mainRun(shopId, cashCode, shifts, dates, converterName=None, dataPath=None, cashUnloadStateId=None,
        statusHost=None, isSendDebugStatus=True, granularity="document", closedShifts=0, changedShifts=False):
    result = False
    configFilePath = sys.path[0] + "/../config/cs/exchangers.ini"
    if config.read(configFilePath):
        nodes = []
        try:
            nodeGetter = __import__('unload.node.%s.nodeGetter' % config.getString('UNLOAD', 'nodeGetter', 'cs30'), fromlist='q')
            nodes = [n for n in nodeGetter.getNodes() if not shopId or shopId == n.id]
        except Exception, e:
            log.error(u'Ошибка получения нод. %s' % e.message)

        workDirectory = config.getString('main', 'workDirectory', '/opt/exchangers/work')
        # generate sales
        if not nodes:
            log.warn(u'Не найдены ноды для магазина с идентификатором %s' % shopId)
        for node in nodes:
            for cash in [c for c in node.cashes if not cashCode or cashCode == c.code]:
                if not applyConverterAndPathOptions(node, cash, converterName, dataPath):
                    continue
                log.info(u'Запускаем ручную выгрузку продаж для кассы с кодом %s из магазина с идентификатором %s' % (cash.code, node.id))
                result = unload(node, workDirectory, shifts=shifts, dates=dates, granularity=granularity, closedShifts=closedShifts, 
                                changedShifts=changedShifts, cashCode=cash.code, cashUnloadStateId=cashUnloadStateId, statusHost=statusHost, 
                                isSendDebugStatus=isSendDebugStatus)
                if not result:
                    return result
    else:
        log.error("Configuration file '%s' not found. The system 'exchangers' is not running." % configFilePath)
        result = False
    return result


def main(argv):
    if not srvLock.lock(False):
        log.error("Can not run service. This service is running")
        os._exit(1)  # выход без выполнения atexit

    parser = optparse.OptionParser()
    parser.add_option("-K", "--cashCode", dest="cashCode", help="Unload for this cashCode")
    parser.add_option("-S", "--shopId", dest="shopId", help="Unload for cashes from this shopId")
    parser.add_option("-s", "--shifts", dest="shifts", help="@-separated shift range")
    parser.add_option("-d", "--dates", dest="dates", help="@-separated dates range. Format: %Y-%m-%dT%H:%M:%S.%f@%Y-%m-%dT%H:%M:%S.%f")
    parser.add_option("-g", "--granularity", dest="granularity", default="document", help="granularity of data: shift or document [default: %default]")
    parser.add_option("-c", "--closedShifts", dest="closedShifts", default=0, type='int', 
                      help="Unload only closed, opened or closed and opened shifts. Use: 0-closed, 1-opened, 2-closedAndOpened  [default: %default]")
    parser.add_option("-C", "--changedShifts", action="store_true", dest="changedShifts", default=False, help="Unload all documents for changed shifts")
    parser.add_option("-v", "--converter", dest="converterName", help=u"Тип конвертора: aif, atol-astor, crystal, ns2000, oracle, shtrih, smag")
    parser.add_option("-p", "--path", dest="dataPath", help=u"Путь куда выгружаются файлы продаж")
    parser.add_option("-u", "--cashUnloadStateId", dest="cashUnloadStateId", help=u"Идентификатор статуса выгрузки продаж")
    parser.add_option("-r", "--statusHost", dest="statusHost", help=u"Хост и порт REST сервера разделенный :")
    parser.add_option("-D", "--debugStatus", action="store_true", dest="isSendDebugStatus", default=True, help=u"Отправлять статусы выгрузки продаж на сервер в debug режиме")
    parser.add_option("-t", "--today", action="store_true", dest="today", default=False, help=u"Выгрузить документы за смены, открытые за текущий день")
    (options, args) = parser.parse_args(argv)

    mainRun(options.shopId, options.cashCode, options.shifts, options.dates, options.converterName, options.dataPath,
            options.cashUnloadStateId, options.statusHost, options.isSendDebugStatus, options.granularity, options.closedShifts, options.changedShifts)
    srvLock.unlock()


if __name__ == '__main__':
    main(sys.argv[1:])
