# -*- coding: UTF-8 -*-

from common import aif_objects as aif, logger
from upload.inserter import InserterError
from upload.converter.smagPlugins.crc16 import *
import xml.etree.ElementTree as ET

errorObjectsCount = 0

class Converter(object):
    def __init__(self, workDirectory, data, node):
        self.workDirectory = workDirectory
        self.data = data
        self.node = node
        self.log = logger.getUploadLogger("Node", node.id)
        plugin = __import__('upload.inserter.%s' % (node.inserter.name), fromlist='Inserter')
        cl = getattr(plugin, 'Inserter')
        self.inserter = cl(node)
        self.listObjects = []

    def run(self):
        global errorObjectsCount
        result = self.inserter.start()

        if result:
            for dictFile in self.data:
                self.log.info(u"Парсинг файла %s" % dictFile)
                try:
                    root = ET.parse(dictFile).getroot()
                except Exception, e:
                    self.log.error(u'Файл %s имеет некорректный формат. %s' % (str(dictFile), str(e)))
                    errorList.append(u'Файл %s имеет некорректный формат. %s' % (str(dictFile), str(e)))
                try:

                    if root.tag == 'ITEMS':

                        measure = {}

                        self.log.info(u'Ищем товары')
                        countObject = 0

                        for goodItem in root.findall('MATERIAL'):
                            ops = {}
                            code = goodItem.find('CODE').text
                            name = goodItem.find('NAME').text
                            price = goodItem.find('PRICE').text
                            vat = goodItem.findtext('NDS','0')
                            obj = aif.Invent().addCmd(inventcode=code, barcode=None, name=name, price=price, lvatcode=[int(vat)])
                            self.listObjects.append(obj)
                            countObject += 1
                            self.sendPackage(False)
                            
                            additionalPrices = []
                            for mrc in goodItem.findall('MRC'):
                                additionalPrices = [item.text for item in mrc.findall('PRICE')]
                                tindex = 1
                                for p in additionalPrices:
                                        obj = aif.AdditionalPrice().addCmd(barcode=code, pricecode=tindex, price=p)
                                        self.listObjects.append(obj)
                                        countObject += 1
                                        tindex += 1
                                        self.sendPackage(False)
                            if len(additionalPrices) > 0:
                                ops["priceoptions"] = { "requireselectprice" : 1 }

                            type = goodItem.findtext('TYPE')
                            if type == '1':
                                ops['inventitemoptions'] = { "ageverify" : 1 }
                            elif type == '2':
                                ops['inventitemoptions'] = { "egaisverify" : 1 }
                            elif type == '3':
                                ops['inventitemoptions'] = { "nopdfegaisverify" : 1 }
                            obj = aif.AddInventOptions(code, options=ops).dict
                            self.listObjects.append(obj)
                            countObject += 1
                            self.sendPackage(False)

                            for barcode in goodItem.findall('BARCODES'):
                                edizm = [item.text for item in barcode.findall('EDIZM')]
                                barcodes = [item.text for item in barcode.findall('EAN')]

                                for a in zip(barcodes, edizm):
                                    if len(a) > 1:

                                        barcode = a[0]
                                        measureName = a[1]

                                        if not measureName in measure.keys():
                                            measure[measureName] = {'code' : crc16(measureName), 'fractional':0}
                                        obj = aif.Barcode().addCmd(code=code, barcode=barcode, measure=measure[measureName]['code'])
                                        self.listObjects.append(obj)
                                        countObject += 1
                                        self.sendPackage(False)
                        self.log.info(u'Загружено объектов в очередь: %s.' % countObject)

                        self.log.info(u'Ищем единицы измерения')
                        countObject = 0
                        for me in measure:
                            countObject += 1
                            unit = aif.Unit().addCmd(unitcode = measure[me]['code'], name = me, fractional = measure[me]['fractional'])
                            self.listObjects.append(unit)

                        self.log.info(u'Загружаем налоги')
                        self.listObjects.append(aif.LVat().addCmd(code=1, rate=18))
                        self.listObjects.append(aif.LVat().addCmd(code=2, rate=10))
                        countObject +=2
                        
                        self.sendPackage(True)
                        self.log.info(u'Загружено объектов в очередь: %s.' % countObject)
                    else:
                        self.log.info(u'Товары не найдены')

                except Exception, e:
                    self.log.error(u'Ошибка в файле: %s. %s' % (str(dictFile), e.message))
                    errorList.append(u'Ошибка в файле: %s. %s' % (str(dictFile), e.message))

        result = self.inserter.stop()
        errorObjectsCount = self.inserter.invalidObjectsCount()
        return result

    def sendPackage(self, anywayAdd):
        if len(self.listObjects) >= self.node.pack_size or anywayAdd:
            self.inserter.add_package(self.listObjects)
            del self.listObjects[:]

def general_method(workDirectory, data, node):
    log = logger.getUploadLogger("Node", node.id)
    result = False
    global errorList
    errorList = []
    try:
        result = Converter(workDirectory, data, node).run()
    except InserterError, e:
        raise
    except Exception, e:
        errorList.append(e)
        log.exception(e)
    return result, errorList

def invalidObjectsCount():
    return errorObjectsCount
