# -*- coding: UTF-8 -*-
'''
Created on 27.09.2012

@author: Starovoytov Danil
'''

from common import aif_objects as dicts
import datetime
import xml.etree.ElementTree as ET

class ShtrihFormatter(object):
    def __init__(self, log, delimiter, cashierName, hotKeyByBarcode, requireSaleRestrict):
        self.log = log
        self.delimiter = delimiter
        self.cashierName = cashierName
        self.hotKeyByBarcode = hotKeyByBarcode
        self.requireSaleRestrict = requireSaleRestrict
        self.clearCommands = {
            '{NO_TOV}': {"command": "clearInventory"},
            '{AUT_S}': {"command": "clearLDiscScheme"},
            '{ACC_S}': {"command": "clearLDiscAccumulate"},     # Очистка накопительных скидок
            '{DIS_C}': {"command": "clearLDiscCard"},
            '{USR}': {"command": "clearMCashUser"},
            # '{NAB_P}': {"command": "clearRole"},
            '{FIX_S}': {"command": "clearLDiscFixed"},
            # '{V_SHK}': None,
            # '{PAY_C}': None,
            '{NAL}': {"command": "NAL"}, # ???????
            '{GR_NL}': {"command": "GR_NL"}, # ??????
            '{PROD}': {"command": "clearConsultant"},
            # '{B_CRD}': None,
            # '{MES}': None,
            # '{VAL}': '', # нет такой
            '{HOT_K}': {"command": "clearHotKey"},
            # '{MOB_OP}': '', # нет такой
            # '{BON}': None,
            '{UNITS}': {"command": "clearUnit"},
            # '{ZAPR}': {"command": "ZAPR"},                    # clearInventTimeResctictions нет в NES
            '{KIT}': {"command": "clearDiscountAction"},      # clearDiscountKits нет в NES
            '{CAT}': [{"command": "clearInventCatalog"}, {"command": "clearCatalog"}],
            '{COM_GR_GF}': [{"command": "clearAspectScheme"}, {"command": "clearAspectValueSet"}],
            '{GOOD_F}': [{"command": "clearAspect"}, {"command": "clearAspectValue"}],
            '{TOV_F}': {"command": "clearAspectValueSetValues"}
        }
        self.formatDescription = {
            "0": lambda line: self.addInventItem(line),
            "#": lambda line: self.addBarcode(line),
            "?": lambda line: self.addAdditionalPriceByInventcode(line),               # also for '{'
            "!": lambda line: self.addLDiscScheme(line),
            "@": lambda line: self.addDiscountsAuto(line),
            "%": lambda line: self.addLDiscCard(line),
            u"№": lambda line: self.addLDiscAccumulate(line),                          # "discCardConditions",
            "^": lambda line: self.addLDiscFixed(line),
            # "~": 'userRights',
            "&": lambda line: self.addMCashUser(line),                                 # "user",
            "+": lambda line: [self.addTaxGroup(line)],                                # "taxes",
            # "*": lambda line: None,                                                   # "taxGroup",
            "'": lambda line: self.addHotKey(line),
            '"': lambda line: self.addHotKeyInvent(line),
            "{": lambda line: self.addAdditionalPriceByBarcode(line),                  # also for '?'
            "U": lambda line: self.addUnit(line),                                      # "measure",
            "Z": lambda line: self.addSellRestrictPeriods(line),                       # "inventTimeRestrictionsByCode",
            "z": lambda line: self.addSellRestrictPeriods(line),                       # "inventTimeRestrictionsByBarcode",
            "O": lambda line: self.addInventItemOptions(line),                         # "inventFlags",
            "o": lambda line: self.addInventItemOptions(line),                         # "barcodeFlags",
            "W": lambda line: self.addRemains(line),                                   # "remainsForAspects",
            "A": lambda line: self.addDiscountAction(line),                            # "discountKit",
            "P": lambda line: self.addTimePeriodForDiscountAction(line),               # "timePeriod",
            "N": lambda line: self.addKitForDiscountAction(line),                      # "discountKitInvent",
            "T": lambda line: self.addKitInventForDiscountAction(line),                # "discountKitInventItem",
            "S": lambda line: self.addDiscForDiscountAction(line),                     # "discountKitInventItemRate",
            "C": lambda line: [self.addInventCatalog(line), self.addCatalog(line)],    # "tmccatalog, catalog",
            "<goods_attr": lambda attrDict: self.addInventItemAttr(attrDict),
            "<group_gf": lambda line: self.addAspectScheme(line),                       # "aspectscheme"
            "<good_feature": lambda line: self.addAspectAndAspectValue(line),           # "aspect". "aspectvalue"
            "{@": lambda line: self.addAspectValueSetAndAspectValueSetValues(line),     # "aspectvalueset", "aspectvaluesetvalues"
            "[": lambda line: self.addConsultant(line)                                 # "consultant"
            # "R": "aspectsForBarcode",
            # "|": 'keyValue'
        }
        self.sprMode = 'RPL'
        self.markedCodes = {}

    def getValuesFromString(self, string):
        array = string.strip().split(self.delimiter)
        for i in range(len(array)):
            if array[i] == '':
                array[i] = None
        for i in range(len(array), 37):
            array.append(None)
        return array

    def toDate(self, string):
        if string:
            return str(datetime.datetime.strptime(string, '%d.%m.%Y' if len(string) > 8 else '%d.%m.%y').date())
        return None

    def getIntParam(self, arg, defaultValue=0):
        try:
            return int(arg)
        except Exception:
            return defaultValue

    # '0'
    def addInventItem(self, line):
        values = self.getValuesFromString(line)
        measurecode = '1'
        if values[12] != '0' and values[12] is not None:
            measurecode = values[12]
        elif values[7] is not None:
            measurecode = str(int(values[7]) + 1)
        if values[16] == '1':
            minprice = 0
            if values[9]:
                maxDiscPerc = float(values[9])
                if maxDiscPerc < 0:
                    minprice = float(values[4])
                elif maxDiscPerc != 0:
                    minprice = float(values[4]) * (100 - maxDiscPerc) / 100

            taracapacity = values[13] if values[13] else 0
            options = {"quantityoptions": {"documentquantlimit": values[14]}}
            tmctype = None
            options["inventitemoptions"] = {}
            if self.requireSaleRestrict:
                options["inventitemoptions"].update({"requiresalerestrict": 1})
            if values[27] == '0002':
                options["inventitemoptions"].update({"rfidverify": 1})
            elif values[27] == '0005':
                options["inventitemoptions"].update({"tobacco": 1})
                tmctype = dicts.Barcode.TOBACCO
            elif values[27] == '1520':
                options["inventitemoptions"].update({"shoes": 1})
                tmctype = dicts.Barcode.SHOES
            elif values[27] == '0003':
                tmctype = dicts.Barcode.MEDICINE
            elif values[27] in ['0004', '0006', '0007', '0008', '0009', '0010', '0011', '0012', '0013', '0014', '444D']:
                tmctype = dicts.Barcode.MARKEDGOODS
            
            self.markedCodes[values[0]] = tmctype

            return dicts.Invent().addCmd(inventcode=values[0], barcode=values[1], name=values[2], price=values[4],
                    remain=values[5], discautoscheme=values[6], taxgroupcode=self.getIntParam(values[10]),
                    articul=values[11], measurecode=measurecode, inventgroup=values[15], minprice=minprice,
                    taracapacity=taracapacity, deptcode=values[8], options=options,
                    paymentobject=self.getIntParam(values[25], None), tmctype=tmctype)
        else:
            return self.addInventGroup(line)

    def addInventGroup(self, line):
        values = self.getValuesFromString(line)
        return dicts.InventGroup().addCmd(groupcode=values[0], parentgroupcode=values[15], groupname=values[2])

    # 'C' - Каталоги товаров
    def addInventCatalog(self, line):
        values = self.getValuesFromString(line)
        return dicts.InventCatalog().addCmd(tmccode=values[0], catalogid=self.getIntParam(values[1]))

    def addCatalog(self, line):
        values = self.getValuesFromString(line)
        return dicts.Catalog().addCmd(catalogid=self.getIntParam(values[1]), catalogname=u'Каталог ' + values[1])

    # '#' - дополнительные штрих-коды товаров
    def addBarcode(self, line):
        values = self.getValuesFromString(line)
        tmcType = self.markedCodes.get(values[0], None)

        minprice = None
        if values[8]:
            minprice = 0
            maxDiscPerc = float(values[8])
            if maxDiscPerc < 0:
                minprice = float(values[4])
            elif maxDiscPerc != 0:
                minprice = float(values[4]) * (100 - maxDiscPerc) / 100

        return dicts.Barcode().addCmd(code=values[0], barcode=values[1], price=values[4],
                name=values[2], measure=values[5], cquant=values[9], tmctype=tmcType, minprice=minprice)

    # '?' - дополнителные цены для товаров с привязкой к коду
    def addAdditionalPriceByInventcode(self, line):
        values = self.getValuesFromString(line)
        return dicts.AdditionalPrice().addCmd(barcode=values[0], pricecode=values[1], price=values[3], name=values[2])

    # '{' - дополнителные цены для товаров с привязкой к штрих-коду
    def addAdditionalPriceByBarcode(self, line):
        values = self.getValuesFromString(line)
        return dicts.AdditionalPrice().addCmd(barcode=values[0], pricecode=values[1], price=values[3], name=values[2])

    # '!' - схема автоматических скидок/надбавок
    def addLDiscScheme(self, line):
        values = self.getValuesFromString(line)
        return dicts.LDiscScheme().addCmd(schemecode=values[0], name=values[1], iscardscheme=values[2])

    # '@' - автоматические скидки/надбавки каждой из схем
    def addDiscountsAuto(self, line):
        values = self.getValuesFromString(line)
        datebeg = self.toDate(values[7])
        dateend = self.toDate(values[8])
        return dicts.DiscountsAuto().addCmd(schemecode=values[0], disccode=values[1], name=values[2],
                disctype=values[3], discmode=values[4], discsize=values[5], rtext=values[6], datebeg=datebeg,
                dateend=dateend, timebeg=values[9], timeend=values[10], daysbeg=values[11], daysend=values[12],
                quantbeg=values[13], quantend=values[14], sumbeg=values[15], sumend=values[16], pricecode=values[17],
                rsummode=values[18], cardgroupcode=values[19], birthdaymode=values[20],
                catalogid=self.getIntParam(values[21], None), catalogquantbeg=values[22], catalogquantend=values[23],
                catalogsumbeg=values[24], catalogsumend=values[25])

    # '%' - дисконтные карты
    def addLDiscCard(self, line):
        values = self.getValuesFromString(line)
        birthday = self.toDate(values[17])
        return dicts.LDiscCard().addCmd(cardnumber=values[0], name=values[1], rtext=values[2], denycard=values[5],
                acdiscasheme=values[12], cardmode=values[13], cardsum=values[3], discasheme=values[4],
                notaccumulate=values[6], positionmode=values[7], ispaycard=values[8], allpositions=values[9],
                msgsheme=values[10], rcount=values[11], bonussum=values[14], bonuscode=values[15],
                groupcode=values[16], birthday=birthday)

    # '№' - условия дисконтных скидок в зависимости от суммы накопления
    def addLDiscAccumulate(self, line):
        values = self.getValuesFromString(line)
        accumulateSPrices = dicts.AccumulateSPrices().addCmd(code=values[12], name=values[13])
        discCardAccumulate = dicts.DiscCardAccumulate().addCmd(disccode=values[0], name=values[1], rtext=values[2],
                discperc=values[3], isbegsum=values[4], begsum=values[5], isendsum=values[6], endsum=values[7],
                isbegrcount=values[8], begrcount=values[9], isendrcount=values[10], endrcount=values[11],
                discscheme=values[12], discschemename=values[13])
        return [accumulateSPrices, discCardAccumulate]

    # '^' - фиксированные скидки
    def addLDiscFixed(self, line):
        values = self.getValuesFromString(line)
        return dicts.LDiscFixed().addCmd(disccode=values[0], name=values[1], disctype=values[2],
                discmode=values[3], discsize=values[4], rtext=values[5], cardpref=values[6], receiptmode=values[7],
                positionmode=values[8], cardonly=values[9], cardsumbeg=values[10], cardsumend=values[11])

    # '&' - пользователи (4.4.5)
    def addMCashUser(self, line):
        values = self.getValuesFromString(line)
        user = values[1] if self.cashierName else values[0]
        mCashUser = dicts.MCashUser()
        mCashUser.addCmd(code=user, login=values[1], name=values[1], password=values[3])
        return mCashUser.addRoleuser(rolecode=values[2])

    # '+' '*' - налоги
    def addTaxGroup(self, line):
        values = self.getValuesFromString(line)
        taxes = [{"name": values[1], "rate": values[3]}]
        return dicts.TaxGroup().addCmd(idtaxgroup=self.getIntParam(values[0]), taxes=taxes)

    # '''
    def addHotKey(self, line):
        values = self.getValuesFromString(line)
        return dicts.HotKey().addCmd(hotkeycode=values[0], hotkeyname=values[1], bybarcode=self.hotKeyByBarcode)

    # '"'
    def addHotKeyInvent(self, line):
        values = self.getValuesFromString(line)
        inventCode = values[6] if self.hotKeyByBarcode else values[1]
        if not inventCode:
            return None
        return dicts.HotKeyInvent().addCmd(hotkeycode=values[0], inventcode=inventCode)

    # 'U' - единицы измерения
    def addUnit(self, line):
        values = self.getValuesFromString(line)
        fract = None
        if values[2] == '1':
            fract = True 
        elif values[2] == '0':
            fract = False
        return dicts.Unit().addCmd(unitcode=values[0], name=values[1], fractional=fract)

    # 'Z' - ограничения продаж по времени для товаров с заданным кодом
    def addSellRestrictPeriods(self, line):
        values = self.getValuesFromString(line)
        sellrestrictperiods = {
            'datestart': values[5],
            'dateend': values[6],
            'daystart': values[7],
            'dayend': values[8],
            'timestart': values[3],
            'timeend': values[4]
        }
        return dicts.AddSellRestrictPeriods(inventcode=values[0], sellrestrictperiods=sellrestrictperiods).dict

    # 'O' - параметры (опции) товара с заданным кодом
    def addInventItemOptions(self, line):
        values = self.getValuesFromString(line)
        ops = {}
        remainsoptions = {
            "traceremaindepartment": self.getIntParam(values[19]),  # Отслеживать остатки в отделе
            "modifyremaindepartment": self.getIntParam(values[20]),  # Корректировать остатки в отделе
            "traceremainback": self.getIntParam(values[21]),  # Отслеживать остатки при возврате
            # "manualremaintrace": self.getIntParam(values[]),
            "traceremainbycode": self.getIntParam(values[23])  # Учет остатков по коду
        }
        quantityoptions = {
            "enabledefaultquantity": self.getIntParam(values[10]),  # Разрешено использовать количество по умолчанию
            "enablequantitymanual": self.getIntParam(values[11]),  # Разрешен ввод количества вручную
            "enablequantitybarcode": self.getIntParam(values[12]),  # Разрешен ввод количества из штрих-кода
            "enablequantityscales": self.getIntParam(values[13]),  # Разрешен ввод количества с весов
            "enablequantitylimit": self.getIntParam(values[14]),  # Запрещена продажа сверх лимита
            "requirequantitymanual": self.getIntParam(values[15]),  # Обязателен ввод количества вручную
            "requirequantityscales": self.getIntParam(values[16]),  # Обязателен ввод количества с весов
            "requirequantitybarcode": self.getIntParam(values[26]),
            "enabledocumentquantitylimit": self.getIntParam(values[27])
        }
        inventitemoptions = {
            "enablebarcodemanual": self.getIntParam(values[2]),  # Разрешен ввод штрих-кода вручную
            "enablebarcodescanner": self.getIntParam(values[3]),  # Разрешен ввод штрих-кода сканером
            "disableinventsale": self.getIntParam(values[4]),  # Запрещена продажа товара
            "disableinventback": self.getIntParam(values[5]),  # Запрещен возврат товара
            "disableinventshow": self.getIntParam(values[6]),  # Не отображать товар в меню кассира
            "disablebackinsale": self.getIntParam(values[24]),  # Запрещен возврат товара в чеке продажи
            "enabledepartmentmanual": self.getIntParam(values[17]),  # Разрешен ввод номера отдела вручную
            "requiredepartmentmanual": self.getIntParam(values[18]),  # Обязательный ввод номера отдела вручную
            "egaisverify": self.getIntParam(values[28]),  # Обязательна проверка в ЕГАИС
            "tobacco": self.getIntParam(values[29]),  # Товар является табачной продукцией
            "visualverify": self.getIntParam(values[31]),  # Необходима визуальная проверка
            "weightcontrolbypass": self.getIntParam(values[32]),  # Пропускать товар при контроле веса
            "lowweight": self.getIntParam(values[33]),  # Легкий товар
            "fuzzyweight": self.getIntParam(values[34]),  # Товар с неточным весом
            "markdownverify": self.getIntParam(values[36])  # Необходимо проверить на сервисе уценки
        }
        if self.requireSaleRestrict:
            inventitemoptions["requiresalerestrict"] = 1

        # discoptions = {
        # "enablediscshemeinput" : self.getIntParam(values[7]), # Разрешен ввод ценовой схемы вручную
        # "requirediscshemeinput": self.getIntParam(values[8]) # Обязательный ввод ценовой схемы вручную
        # }
        priceoptions = {
            "requireselectprice": self.getIntParam(values[9]),  # Обязательный выбор цены товара из списка
            "enableexcisemarkprice": self.getIntParam(values[30])  # Получать цену из кода маркировки табачной продукции
        }

        ops["remainsoptions"] = remainsoptions
        ops["quantityoptions"] = quantityoptions
        ops["inventitemoptions"] = inventitemoptions
        #ops["discoptions"] = discoptions
        ops["priceoptions"] = priceoptions

        obj = dicts.AddInventOptions(inventcode=values[0], options=ops).dict
        obj["invent"]["extendedoptions"] = values[35]
        obj["invent"]["taramode"] = self.getIntParam(values[25]) if len(values) > 24 else 0

        return obj

    # 'W' - остатки
    def addRemains(self, line):
        values = self.getValuesFromString(line)
        datetime = self.toDate(values[3]) + 'T' + values[4]
        return dicts.AddRemain(inventcode=values[0], barcode=values[1], date=datetime,
                               remain=values[4], deptcode=values[5]).dict

    # 'A' - дисконтные акции
    def addDiscountAction(self, line):
        values = self.getValuesFromString(line)
        return dicts.DiscountAction().addCmd(iddiscountaction=values[0], name=values[1])

    # 'P' - период действия акции
    def addTimePeriodForDiscountAction(self, line):
        values = self.getValuesFromString(line)
        datestart = self.toDate(values[2])
        dateend = self.toDate(values[3])

        return dicts.TimePeriod().addCmd(idtimeperiod=values[0], iddiscountaction=values[1], datestart=datestart,
                dateend=dateend, timestart=values[4], timeend=values[5], daystart=values[6], dayend=values[7])

    # 'N' - комплекты (наборов товара)
    def addKitForDiscountAction(self, line):
        values = self.getValuesFromString(line)
        return dicts.Kit().addCmd(iddiscountaction=values[2], idkit=values[0], name=values[1],
                kittype=values[3], maxtimes=values[4])

    # 'T' - позиции комплекта (набора)
    def addKitInventForDiscountAction(self, line):
        values = self.getValuesFromString(line)
        return dicts.KitInvent().addCmd(idkitgoods=values[0], idkit=values[1], inventcode=values[2], quantity=values[3])

    # 'S' - скидки на позицию в комплекте(наборе)
    def addDiscForDiscountAction(self, line):
        values = self.getValuesFromString(line)
        value = float(values[4])
        if values[2] == 2:  # надбавка
            value = value * (-1)
        kind = None
        if values[3] == '1':
            kind = 'percent'
        elif values[3] == '2':
            kind = 'sum'
        return dicts.KitDisc().addCmd(idkitdisc=values[0], idkitgoods=values[1], type=kind, value=value)

    # '<goods_attr ' - дополнительные реквизиты товара
    def addInventItemAttr(self, attrDict):
        result = []
        for tmcCode in attrDict:
            obj = {}
            for attr in attrDict[tmcCode]:
                attrValue = attrDict[tmcCode][attr]
                if attr == '22':
                    obj['alctypecode'] = attrValue
                elif attr == '23':
                    obj['taracapacity'] = attrValue
                elif attr == '24':
                    obj['options'] = {}
                    obj['options']['inventitemoptions'] = {}
                    if attrValue == '0':
                        obj['options']['inventitemoptions']['egaisverify'] = 1
                    else:
                        obj['options']['inventitemoptions']['nopdfegaisverify'] = 1
                    if self.requireSaleRestrict:
                        obj['options']['inventitemoptions']["requiresalerestrict"] = 1
                elif attr == '25':
                    obj['inn'] = attrValue
                elif attr == '26':
                    obj['kpp'] = attrValue
                elif attr == '27':
                    obj['alcoholpercent'] = attrValue
                elif attr == '37':
                    obj['minretailprice'] = attrValue

            dictionary = dicts.UpdateInventAttr(inventcode=tmcCode).dict
            dictionary['invent'].update(obj)
            result.append(dictionary)
        return result

    # '<group_gf' - схема разрезов
    def addAspectScheme(self, line):
        '''
        Стоит помнить, что у нас в БД код схемы разрезов - целое, а у Штриха - строка
        '''
        xmlCommand = ET.fromstring(line)
        code = int(xmlCommand.attrib['code'])
        name = xmlCommand.attrib['name']
        return dicts.AspectScheme().addCmd(aspectschemecode=code, aspectschemename=name)

    # '<good_feature' - характеристики разрезов и значения характеристик разрезов
    def addAspectAndAspectValue(self, line):
        result = []
        xmlCommand = ET.fromstring(line)
        aspectCode = xmlCommand.attrib['code']
        aspectName = xmlCommand.attrib['name']
        for node in map(lambda x: x.strip(), xmlCommand.text.split('\n')):
            if not len(node):
                continue
            values = map(lambda x: x.strip(), node.split(';'))
            result.append(dicts.AspectValue().addCmd(aspectvaluecode=values[0],
                    aspectvaluename=values[1], aspectcode=aspectCode))
        result.append(dicts.Aspect().addCmd(aspectcode=aspectCode, aspectname=aspectName,
                aspectorder=None, aspectschemecode=None, aspectvaluedesc=None))
        return result

    # '{@' - цены и остатки разрезов характеристик товара и разрезы характеристик товара
    def addAspectValueSetAndAspectValueSetValues(self, line):
        result = []
        lines = filter(None, line.split('\n'))
        values = self.getValuesFromString(lines[0])
        tmcCode = values[0]
        productSectionCode = values[1]
        name = values[3]
        values = self.getValuesFromString(lines[-1])
        price = float(values[2])
        remain = float(values[3])

        for line in lines[1:-1]:
            values = self.getValuesFromString(line)
            aspectCode = values[1]
            aspectValueCode = values[2]
            result.append(dicts.AspectValueSetValues().addCmd(aspectCode, aspectValueCode, aspectvaluesetcode=None,
                    productsectioncode=productSectionCode, tmc_code=tmcCode))
        if self.sprMode == 'ADD':
            result.append(dicts.AspectValueSet().updateCmd(None, None, cquant=None, price=price, tmc_code=tmcCode,
                    measure=None, remain=remain, name=name, remaindate=None,
                    packingprice=None, productsectioncode=productSectionCode))
        else:
            result.append(dicts.AspectValueSet().addCmd(None, None, cquant=None, price=price, tmc_code=tmcCode,
                    measure=None, remain=remain, name=name, remaindate=None,
                    packingprice=None, productsectioncode=productSectionCode))
        tmcDictionary = dicts.UpdateInventAttr(inventcode=tmcCode).dict
        tmcDictionary['invent'].update({'aspectusecase': 3})
        result.append(tmcDictionary)
        return result

    # '[' - продавец(консультант)
    def addConsultant(self, line):
        values = self.getValuesFromString(line)
        return dicts.Consultant().addCmd(code=values[0], password=values[1], name=values[2])

    # '|'
    def setSprMode(self, sprMode):
        self.sprMode = sprMode
