# -*- coding: utf-8 -*-
import base64
import os
import signal
import sys
sys.path[0] = os.path.normpath('%s/../../..' % sys.path[0])

from common import logger
logger.location = 'cs'

import httplib2
from httplib import INTERNAL_SERVER_ERROR
from common import config, macros, statuses
from common.statuses import upload as st
from bottle import route, run, response, request, debug
import json
import traceback
from datetime import datetime

log = logger.getUploadLogger("RestStatusService")
log.info("exchange-rest-service start for cs")

restServiceHost = 'localhost'
restServicePort = 8998

jsonHeader = {'Content-Type':'application/json', "Accept": "application/json"}
requestTimeout = 60

_lastTimeNodeRead = None
_traceFileDirs = {}

def getParamValue(exchange, key, defaultValue=None):
    l = filter(lambda param: param.get('key', None) == key, exchange.get('params', []))
    value = l[0].get('value', defaultValue) if len(l) > 0 else defaultValue
    return value

def _getTraceFilePath(shopCode, format=None):
    global _lastTimeNodeRead, _traceFileDirs
    timeoutNodeReread = config.getInt('UPLOAD', 'timeoutNodeReread', 300)
    if _lastTimeNodeRead is None or (datetime.now() - _lastTimeNodeRead).seconds > timeoutNodeReread:
        try:
            _traceFileDirs.clear()
            http = httplib2.Http(timeout=requestTimeout)
            response, content = http.request(httpCSRestUri, 'GET', headers=jsonHeader)
            if response.status == 200:
                shops = json.loads(content, encoding='utf8')
                for shop in shops:
                    nodeShopCode = unicode(shop.get('shopCode'))
                    shopId = unicode(shop.get('id'))
                    clusterId = unicode(shop.get('clusterId', ''))
                    _traceFileDirs[shopId] = {}
                    for exchange in shop.get('exchanges', []):
                        traceFilePath = str(getParamValue(exchange, 'traceFileDir', u'/tmp'))
                        if traceFilePath:
                            traceFileDir = unicode(macros.applyMacros(traceFilePath, log, shopcode=nodeShopCode, shopid=shopId, clusterid=clusterId))
                            _traceFileDirs[shopId][exchange.get('type', 'DEFAULT')] = traceFileDir
        except Exception:
            statuses.setUploadStatus(st.ERROR_READ_NODES)
            log.error(u'Ошибка считывания нод.')
            log.error(traceback.format_exc())
        _lastTimeNodeRead = datetime.now()
    if format:
        return _traceFileDirs.get(shopCode, {}).get(format)
    # если у статуса не известен формат справочника, то путь для создания файла со статусами будем брать из первого попавшегося формата, который указан для магазина
    return _traceFileDirs.get(shopCode, {}).values()[0]

def _createReport(rowNode, resultStatus):
    if resultStatus is None :
        raise Exception(u'resultStatus does not may None')
    if rowNode:
        timeOfOperation = unicode(datetime.fromtimestamp(rowNode.get('time') / 1000))
        node = {u'ИД':rowNode.get('id'), u'Время':timeOfOperation, u'Статус':rowNode.get('status'), u'Сообщение':rowNode.get('messageRu'), \
                u'Код магазина': rowNode.get('shopCode')}
    else:
        return u'неправильный объект'
    
    if rowNode.get('dictCSLoads') is not None:
        # This is dictBackOffice
        node[u'Название файла'] = rowNode.get('fileName')
        node[u'Формат выгрузки'] = rowNode.get('format')
        if rowNode.get('dictCSLoads'):
            childs = []
            for dictCSLoad in rowNode.get('dictCSLoads'):
                childs.append(_createReport(dictCSLoad, resultStatus))
            node[u'Загрузки на КС'] = childs
        else:
            resultStatus.append(u'%s: выгрузка справочника из АСТУ: %s\n' % (timeOfOperation, rowNode.get('messageRu')))
            
    elif rowNode.get('dictGenerates') is not None:
        # This is dictCashLoad
        unloadElementExist = False
        if rowNode.get('unloadElement'):
            element = rowNode.get('unloadElement')
            node[u'Тип выгрузки'] = u'%s: %s%s' % (element.get('name'), element.get('description'), u' (глобальная)' if element.get('global') else u'')
            unloadElementExist = True
        if rowNode.get('dictGenerates'):
            childs = []
            for dictGenerate in rowNode.get('dictGenerates'):
                childs.append(_createReport(dictGenerate, resultStatus))
            node[u'Генерации справочников'] = childs
        else:
            resultStatus.append(u'%s: загрузка %s справочника на КС: %s\n' % (timeOfOperation, \
                                                                              rowNode.get('unloadElement', {}).get('name') if unloadElementExist else u'не указанного', \
                                                                              rowNode.get('messageRu')))
            
    elif rowNode.get('dictCashLoads') is not None:
        # This is dictGenerate
        node[u'Генерация:'] = u'полная' if rowNode.get('full') else u'инкрементальная'
        unloadElementExist = False
        if rowNode.get('unloadElement'):
            element = rowNode.get('unloadElement')
            node[u'Тип выгрузки'] = u'%s: %s%s' % (element.get('name'), element.get('description'), u' (глобальная)' if element.get('global') else u'')
            unloadElementExist = True 
        if rowNode.get('dictCashLoads'):
            childs = []
            for dictCashLoad in rowNode.get('dictCashLoads'):
                childs.append(_createReport(dictCashLoad, resultStatus))
            node[u'Загрузки на кассы'] = childs
        else:
            resultStatus.append(u'%s: генерация %s справочника на КС: %s\n' % (timeOfOperation, \
                                                                               rowNode.get('unloadElement', {}).get('name') if unloadElementExist else u'не указанного', \
                                                                               rowNode.get('messageRu')))
            
    elif rowNode.get('cashCode'):
        # This is dictCashLoad
        node[u'Код кассы'] = rowNode.get('cashCode')
        resultStatus.append(u'%s: выгрузка %s справочника на %s кассу: %s\n' % (timeOfOperation, rowNode.get('dictType'), rowNode.get('cashCode'), rowNode.get('messageRu')))
             
    return node


def stop(signum, frame):
    pass
signal.signal(signal.SIGTERM, stop)

if not config.read(sys.path[0] + "/../config/cs/exchangers.ini"):
    log.error(u"Configuration file 'exchangers.ini' not found. The system 'exchangers' is not running.")
    exit(1)
    
cs_host= 'localhost'
cs_port='8080'
cs_login = 'admin'
cs_password = 'admin'
cs_prefix='/CSrest/rest'
cs_host = config.getString('CS', 'node.host', cs_host)
cs_port = config.getString('CS', 'node.port', cs_port)
cs_login = config.getString('CS', 'login', cs_login)
cs_password = config.getString('CS', 'passwd', cs_password)
cs_prefix = config.getString('CS', 'node.prefix', cs_prefix)

auth = base64.b64encode('{0}:{1}'.format(cs_login, cs_password).encode('utf-8')).decode()
jsonHeader['Authorization'] = 'Basic %s' % auth

base_url='http://'+cs_host+':'+cs_port+cs_prefix    
httpCSRestUri = base_url+'/shops'    

def default(obj):
    """Default JSON serializer."""

    if isinstance(obj, datetime):
        return unicode(obj)
    return obj

def createTraceFile(traceFileName):
    if not os.path.isdir(os.path.dirname(traceFileName)):
        os.makedirs(os.path.dirname(traceFileName))
    return open(os.path.normpath(traceFileName), 'w')

@route('/exchanger/dict/status', method='POST')
def restStatus():
    response.content_type = 'application/json'
    body = json.loads(request.body.read(), encoding='utf8')
    result = {u'success': False}
    try:
        resultStatus = []
        reportDict = _createReport(body, resultStatus)
        shopCode = body.get('shopCode')
        format = body.get('format')
        traceFileName = u"%s/%s_%s.trace.log" % (_getTraceFilePath(shopCode, format), body['fileName'], body['id'])
        traceStream = createTraceFile(traceFileName)
        for s in resultStatus:
            traceStream.write(s.encode('utf8'))
        traceStream.write(u'\n\n\n')

        jsonDecodeString = json.dumps(reportDict, sort_keys=False, indent=4, default=default, ensure_ascii=False)

        traceStream.write(jsonDecodeString.encode('utf8'))
        traceStream.close()
        result = {u'success': True}
    except Exception:
        errorStr = u''
        if isinstance(body, dict):
            errorStr = u'Не удалось обновить статус для справочника %s и id:%s:' % (body.get('fileName'), body.get('id'))
        else:
            if not isinstance(body, unicode):
                body = body.decode('utf8')
            errorStr = u'В запросе был передан некорректный объект: %s' % body
        log.error(errorStr)
        log.error(traceback.format_exc())
        response.status = INTERNAL_SERVER_ERROR
        result = {u'success': False, u'description':errorStr}
    return json.dumps(result, ensure_ascii=False)


@route('/exchanger/dict/status/remove', method='POST')
def remove():
    response.content_type = 'application/json'
    body = json.loads(request.body.read(), encoding='utf8')
    result = {u'success': False}
    try:
        body = body.get('dictBackOffice')
        shopCode = body.get('shopCode')
        fileName = body.get('fileName')
        statusId = body.get('id')
        format = body.get('format')
        traceFileName = u"%s/%s_%s.trace.log" % (_getTraceFilePath(shopCode, format), fileName, statusId)
        if(not os.path.exists(traceFileName)):
            description = u'Не удалось удалить trace-файл для %s: %s - файл не найден в директории' % (fileName, statusId)
            log.error(description)
            result[u'description'] = description
        else:
            traceStream = createTraceFile(traceFileName)
            traceStream.write((u'Не было найдено изменений для %s: %s' % (fileName, statusId)).encode('utf8'))
            traceStream.write('\n')

            traceStream.close()
            result = {u'success': True}
    except Exception:
        errorStr = ''
        if isinstance(body, dict):
            errorStr = u'Не удалось удалить trace-файл для %s и id:%s:' % (body.get('fileName'), body.get('id'))
        else:
            if not isinstance(body, unicode):
                body = body.decode('utf8')
            errorStr = u"В запросе был передан некорректный объект: %s" % body
        log.error(errorStr)
        log.error(traceback.format_exc())
        response.status = INTERNAL_SERVER_ERROR
        result = {u'success': False, u'description':errorStr}
    return json.dumps(result)

def start():
    try:
        run(host=restServiceHost, port=restServicePort)
    except Exception:
        log.error(traceback.format_exc())


if __name__ == '__main__':
    start()
